# coding: utf-8
r"""
Small-step walks

Examples from the paper "Hypergeometric expressions for generating functions
of walks with small steps in the quarter plane" by Alin Bostan, Frédéric
Chyzak, Mark van Hoeij, Manuel Kauers and Lucien Pech (*European Journal of
Combinatorics*, 2017; arXiv:1606.02982 [math.CO];
<http://specfun.inria.fr/chyzak/ssw/index.html>).

For the rational functions rat[1], ..., rat[19], the task consists in computing
a telescoper wrt Du and Dv. Interesting settings are when x and y are both set
to 1 or one of them is set to 0 and the other is left symbolic. For example::

    sage: from ore_algebra import OreAlgebra
    sage: A.<Du,Dv,Dt> = OreAlgebra(ZZ['u','v','t'].fraction_field(), 'Du', 'Dv', 'Dt')

    sage: from ore_algebra.examples import ssw
    sage: q = A.base_ring()(ssw.rat[1](x=1,y=1))
    sage: ct1 = A.ideal([q*D - D(q) for D in (Du,Dv,Dt)]).ct(Dv)[0]
    sage: ct2 = ct1[0].parent().ideal(ct1).ct(Du)[0]
    sage: ct2
    [(16*t^4 - t^2)*Dt^3 + (128*t^3 + 8*t^2 - 6*t)*Dt^2 + (224*t^2 + 28*t - 6)*Dt + 64*t + 12]
    sage: ct2 == [ssw.dop[1,1,1]]
    True

After obtaining the telescopers, we can use them to compute connection matrices
that encode information on the asymptotics of the corresponding lattice walks::

    sage: mat = ssw.dop[5,1,1].numerical_transition_matrix([0, 1/3+i/10, 1/3]) # (1 s)
    sage: (mat*vector([0, 0, 1, 3, 7]))[0]
    [...] + [0.50000...]*I

    sage: ssw.dop[1, 0, 0].numerical_transition_matrix([0, 1/4])
    [ [-206.5126691369...] + [...]*I            [16.000000000000...] + [...]*I  [1.20938909474579...] + [...]*I]
    [ [1489.126691369...] + [...]*I             [-128.00000000000...] + [...]*I [3.9061090525420...] + [...]*I]
    [[1925.57548817...] + [-1536.00000000...]*I [768.000000000...] + [...]*I    [-124.21690469554...] + [128.000000000000...]*I]

TESTS::

    sage: for i in [2, 5, 12, 18]: # long time (1 min 21 s)
    ....:     for xx, yy in [(0, 1), (1, 0)]:
    ....:         ssw.test_ct(i, xx, yy)

    sage: for i in [1..19]: # not tested (9 min 31 s)
    ....:     for xx in [0, 1]:
    ....:         for yy in [0, 1]:
    ....:             ssw.test_ct(i, xx, yy)

Plus an example provided by Bruno Salvy, also related to small-step walks (not
double-checked)::

    sage: ssw.aux_dop.numerical_solution(ssw.aux_ini, [0,1])
    [10.662510694...]

"""

from sage.rings.all import Frac, Integer, PolynomialRing, ZZ, QQ
from ore_algebra import DifferentialOperators, OreAlgebra

Pols, (u, v, x, y, t) = PolynomialRing(ZZ, ('u', 'v', 'x', 'y', 't')).objgens()
DiffOps, t, Dt = DifferentialOperators(QQ, 't')

# Data adapted from https://specfun.inria.fr/chyzak/ssw/ct-P.mpl
# Indexed from 1 on in order to be consistent with the paper

rat = {}

rat[1] = (-1+u**2-u**2*v**2+v**2)/(-u*v+u*v**2*y+u**2*v*x-u**2*v**2*x*y+t*v
        -t*v**2*y-t*v*x*u+t*v**2*x*u*y+t*u-t*u*y*v-t*u**2*x+t*u**2*x*y*v
        +t*u**2*v-t*u**2*v**2*y-t*u**3*v*x+t*u**3*v**2*x*y+t*u*v**2-t*u*v**3*y
        -t*u**2*v**2*x+t*u**2*v**3*x*y)

rat[2] = (-1+u**2-u**2*v**2+v**2)/(-u*v+u*v**2*y+u**2*v*x-u**2*v**2*x*y+t
        -t*y*v-t*x*u+t*x*u*y*v+t*u**2-t*u**2*y*v-t*u**3*x+t*u**3*x*y*v
        +t*u**2*v**2-t*u**2*v**3*y-t*u**3*v**2*x+t*u**3*v**3*x*y+t*v**2
        -t*v**3*y-t*v**2*x*u+t*v**3*x*u*y)

rat[3] = (-1+u**2-u**2*v**2+v**2)/(t+t*u*v**2-t*u**2*v**2*x-u**2*v**2*x*y
        -t*u*y*v-t*u*v**3*y+t*u**2*v**2-u*v+t*u+u**2*v*x-t*u**2*x+u*v**2*y
        +t*u**2*x*y*v+t*u**2*v**3*x*y+t*u**2+t*v**2-t*u**2*y*v-t*u**2*v**3*y
        -t*v**2*x*u-t*u**3*v**2*x-t*y*v-t*v**3*y-t*x*u-t*u**3*x+t*x*u*y*v
        +t*u**3*x*y*v+t*u**3*v**3*x*y+t*v**3*x*u*y)

rat[4] = (-1+u**2-u**2*v**2+v**2)/(t+t*u**2*v+t*u*v**2-t*v*x*u-t*u**3*v*x
        -t*u**2*v**2*x-u**2*v**2*x*y-t*u*y*v-t*u**2*v**2*y-t*u*v**3*y
        +t*u**2*v**2-u*v+t*v+t*u+u**2*v*x-t*u**2*x+u*v**2*y-t*v**2*y
        +t*v**2*x*u*y+t*u**2*x*y*v+t*u**3*v**2*x*y+t*u**2*v**3*x*y+t*u**2
        +t*v**2-t*u**2*y*v-t*u**2*v**3*y-t*v**2*x*u-t*u**3*v**2*x-t*y*v
        -t*v**3*y-t*x*u-t*u**3*x+t*x*u*y*v+t*u**3*x*y*v+t*u**3*v**3*x*y
        +t*v**3*x*u*y)

rat[5] = (-1+u**4-u**3*v**2+u*v**2)/(t+t*u*v**2-t*u**2*v**2*x
        -u**2*v**2*x*y-t*u*v**3*y-u*v+u**2*v*x+u*v**2*y+t*u**2*v**3*x*y
        +2*t*u**2-2*t*u**2*y*v-t*y*v-u**5*t*x+u**3*v**2*y+t*u**3*v**2
        -t*x*u-2*t*u**3*x+t*x*u*y*v+2*t*u**3*x*y*v-t*u**3*v**3*y+u**4*v*x
        -t*u**4*v**2*x-u**4*v**2*x*y-u**4*t*y*v-u**3*v+u**4*t+u**5*t*x*y*
        v+t*u**4*v**3*x*y)

rat[6] = (-1+u**4-u**3*v**2+u*v**2)/(t+2*t*u**2*v+t*u*v**2-t*v*x*u
        -2*t*u**3*v*x-t*u**2*v**2*x-u**2*v**2*x*y-2*t*u**2*v**2*y-t*u*v**3*y
        -u*v+t*v+u**2*v*x+u*v**2*y-t*v**2*y+t*v**2*x*u*y+2*t*u**3*v**2*x*y
        +t*u**2*v**3*x*y+2*t*u**2-2*t*u**2*y*v-t*y*v-u**5*t*x+u**3*v**2*y
        +t*u**3*v**2-t*x*u-2*t*u**3*x+t*x*u*y*v+2*t*u**3*x*y*v-t*u**3*v**3*y
        +u**4*v*x-t*u**4*v**2*x-u**4*v**2*x*y-u**4*t*y*v+u**4*t*v-u**3*v
        +u**4*t+u**5*t*x*y*v+t*u**4*v**3*x*y-u**5*t*x*v-u**4*t*v**2*y
        +u**5*t*x*v**2*y)

rat[7] = (-u-1+u**4+u**3-u**3*v**2+u*v**2)/(t+t*u*v**2-t*u**2*v**2*x
        -u**2*v**2*x*y-2*t*u*y*v-t*u*v**3*y+u**2*v**2*y+u**3*v*x+t*u**2*v**2
        -u*v-u**2*v+2*t*u+u**2*v*x-2*t*u**2*x+u*v**2*y+2*t*u**2*x*y*v
        +t*u**2*v**3*x*y-u**3*v**2*x*y+3*t*u**2-3*t*u**2*y*v-t*u**2*v**3*y
        -t*u**3*v**2*x-t*y*v-2*t*u**4*x-u**5*t*x+u**3*v**2*y+t*u**3*v**2-t*x*u
        -3*t*u**3*x+t*x*u*y*v+3*t*u**3*x*y*v+t*u**3*v**3*x*y+2*t*u**3
        -2*t*u**3*y*v-t*u**3*v**3*y+u**4*v*x-t*u**4*v**2*x-u**4*v**2*x*y
        -u**4*t*y*v-u**3*v+u**4*t+u**5*t*x*y*v+t*u**4*v**3*x*y+2*t*u**4*x*y*v)

rat[8] = (-u-1+u**4+u**3-u**3*v**2+u*v**2)/(t+2*t*u**2*v+t*u*v**2-t*v*x*u
        -2*t*u**3*v*x-t*u**2*v**2*x-u**2*v**2*x*y-2*t*u*y*v-2*t*u**2*v**2*y
        -t*u*v**3*y+u**2*v**2*y+u**3*v*x+t*v*u+t*u**3*v+t*u**2*v**2-u*v-u**2*v
        +t*v+2*t*u+u**2*v*x-2*t*u**2*x+u*v**2*y-t*v**2*y+t*v**2*x*u*y
        +2*t*u**2*x*y*v+2*t*u**3*v**2*x*y+t*u**2*v**3*x*y-u**3*v**2*x*y
        +3*t*u**2-t*v**2*u*y-3*t*u**2*y*v-t*u**3*v**2*y-t*u**2*v**3*y
        -t*u**2*x*v-t*u**3*v**2*x-t*y*v-2*t*u**4*x-u**5*t*x+u**3*v**2*y
        +t*u**3*v**2-t*x*u-3*t*u**3*x+t*x*u*y*v+3*t*u**3*x*y*v+t*u**3*v**3*x*y
        +2*t*u**3-2*t*u**3*y*v-t*u**3*v**3*y+u**4*v*x-t*u**4*v**2*x
        -u**4*v**2*x*y-u**4*t*y*v+u**4*t*v-u**3*v+u**4*t+u**5*t*x*y*v
        +t*u**4*v**3*x*y-u**5*t*x*v-u**4*t*v**2*y+u**5*t*x*v**2*y
        +2*t*u**4*x*y*v-t*u**4*x*v+t*u**2*x*v**2*y+t*u**4*x*v**2*y)

rat[9] = (-u-1+u**4+u**3-u**4*v**2+v**2)/(t+t*u*v**2-t*u**2*v**2*x
        -u**2*v**2*x*y-2*t*u*y*v-t*u*v**3*y+u**2*v**2*y+u**3*v*x+2*t*u**2*v**2
        -u*v-u**2*v+2*t*u+u**2*v*x-2*t*u**2*x+u*v**2*y+2*t*u**2*x*y*v
        +t*u**2*v**3*x*y-u**3*v**2*x*y+3*t*u**2+t*v**2-3*t*u**2*y*v
        -2*t*u**2*v**3*y-t*v**2*x*u-2*t*u**3*v**2*x-t*y*v-t*v**3*y-2*t*u**4*x
        -u**5*t*x+u**3*v**2*y+t*u**3*v**2-t*x*u-3*t*u**3*x+t*x*u*y*v
        +3*t*u**3*x*y*v+2*t*u**3*v**3*x*y+t*v**3*x*u*y+2*t*u**3-2*t*u**3*y*v
        -t*u**3*v**3*y+u**4*v*x-t*u**4*v**2*x-u**4*v**2*x*y-u**4*t*y*v
        +t*u**4*v**2-u**3*v+u**4*t+u**5*t*x*y*v+t*u**4*v**3*x*y-t*u**4*v**3*y
        -u**5*t*x*v**2+2*t*u**4*x*y*v+u**5*t*x*v**3*y)

rat[10] = (-u-1+u**4+u**3-u**4*v**2+v**2)/(t+2*t*u**2*v+t*u*v**2-t*v*x*u
        -2*t*u**3*v*x-t*u**2*v**2*x-u**2*v**2*x*y-2*t*u*y*v-2*t*u**2*v**2*y
        -t*u*v**3*y+u**2*v**2*y+u**3*v*x+t*v*u+t*u**3*v+2*t*u**2*v**2-u*v
        -u**2*v+t*v+2*t*u+u**2*v*x-2*t*u**2*x+u*v**2*y-t*v**2*y+t*v**2*x*u*y
        +2*t*u**2*x*y*v+2*t*u**3*v**2*x*y+t*u**2*v**3*x*y-u**3*v**2*x*y
        +3*t*u**2+t*v**2-t*v**2*u*y-3*t*u**2*y*v-t*u**3*v**2*y-2*t*u**2*v**3*y
        -t*v**2*x*u-t*u**2*x*v-2*t*u**3*v**2*x-t*y*v-t*v**3*y-2*t*u**4*x
        -u**5*t*x+u**3*v**2*y+t*u**3*v**2-t*x*u-3*t*u**3*x+t*x*u*y*v
        +3*t*u**3*x*y*v+2*t*u**3*v**3*x*y+t*v**3*x*u*y+2*t*u**3-2*t*u**3*y*v
        -t*u**3*v**3*y+u**4*v*x-t*u**4*v**2*x-u**4*v**2*x*y-u**4*t*y*v
        +t*u**4*v**2+u**4*t*v-u**3*v+u**4*t+u**5*t*x*y*v+t*u**4*v**3*x*y
        -t*u**4*v**3*y-u**5*t*x*v-u**4*t*v**2*y+u**5*t*x*v**2*y-u**5*t*x*v**2
        +2*t*u**4*x*y*v-t*u**4*x*v+t*u**2*x*v**2*y+t*u**4*x*v**2*y
        +u**5*t*x*v**3*y)

rat[11] = (-u+u**3-u**4*v**2-u**3*v**2+u*v**2+v**2)/(-u**2*v+u**2*v**2*y
        +u**3*v*x-u**3*v**2*x*y+t*u**2-t*u**2*y*v-t*u**3*x+t*u**3*x*y*v
        +t*u**3*v**2-t*u**3*v**3*y-t*u**4*v**2*x+t*u**4*v**3*x*y+t*u**2*v**2
        -t*u**2*v**3*y-t*u**3*v**2*x+t*u**3*v**3*x*y+t*u*v**2-t*u*v**3*y
        -t*u**2*v**2*x+t*u**2*v**3*x*y)

rat[12] = (-u+u**3-u**4*v**2-u**3*v**2+u*v**2+v**2)/(t*u*v**2-t*u**2*v**2*x
        -t*u*v**3*y+u**2*v**2*y+u**3*v*x+t*v*u+t*u**3*v+t*u**2*v**2-u**2*v
        +t*u**2*v**3*x*y-u**3*v**2*x*y+t*u**2-t*v**2*u*y-t*u**2*y*v
        -t*u**3*v**2*y-t*u**2*v**3*y-t*u**2*x*v-t*u**3*v**2*x+t*u**3*v**2
        -t*u**3*x+t*u**3*x*y*v+t*u**3*v**3*x*y-t*u**3*v**3*y-t*u**4*v**2*x
        +t*u**4*v**3*x*y-t*u**4*x*v+t*u**2*x*v**2*y+t*u**4*x*v**2*y)

rat[13] = (-1+u**4-u**4*v**2-u**3*v**2+u*v**2+v**2)/(t+t*u*v**2-t*u**2*v**2*x
        -u**2*v**2*x*y-t*u*v**3*y+2*t*u**2*v**2-u*v+u**2*v*x+u*v**2*y
        +t*u**2*v**3*x*y+2*t*u**2+t*v**2-2*t*u**2*y*v-2*t*u**2*v**3*y
        -t*v**2*x*u-2*t*u**3*v**2*x-t*y*v-t*v**3*y-u**5*t*x+u**3*v**2*y
        +t*u**3*v**2-t*x*u-2*t*u**3*x+t*x*u*y*v+2*t*u**3*x*y*v
        +2*t*u**3*v**3*x*y+t*v**3*x*u*y-t*u**3*v**3*y+u**4*v*x-t*u**4*v**2*x
        -u**4*v**2*x*y-u**4*t*y*v+t*u**4*v**2-u**3*v+u**4*t+u**5*t*x*y*v
        +t*u**4*v**3*x*y-t*u**4*v**3*y-u**5*t*x*v**2+u**5*t*x*v**3*y)

rat[14] = (-1+u**4-u**4*v**2-u**3*v**2+u*v**2+v**2)/(t+2*t*u**2*v+t*u*v**2
        -t*v*x*u-2*t*u**3*v*x-t*u**2*v**2*x-u**2*v**2*x*y-2*t*u**2*v**2*y
        -t*u*v**3*y+2*t*u**2*v**2-u*v+t*v+u**2*v*x+u*v**2*y-t*v**2*y
        +t*v**2*x*u*y+2*t*u**3*v**2*x*y+t*u**2*v**3*x*y+2*t*u**2+t*v**2
        -2*t*u**2*y*v-2*t*u**2*v**3*y-t*v**2*x*u-2*t*u**3*v**2*x-t*y*v
        -t*v**3*y-u**5*t*x+u**3*v**2*y+t*u**3*v**2-t*x*u-2*t*u**3*x+t*x*u*y*v
        +2*t*u**3*x*y*v+2*t*u**3*v**3*x*y+t*v**3*x*u*y-t*u**3*v**3*y+u**4*v*x
        -t*u**4*v**2*x-u**4*v**2*x*y-u**4*t*y*v+t*u**4*v**2+u**4*t*v-u**3*v
        +u**4*t+u**5*t*x*y*v+t*u**4*v**3*x*y-t*u**4*v**3*y-u**5*t*x*v
        -u**4*t*v**2*y+u**5*t*x*v**2*y-u**5*t*x*v**2+u**5*t*x*v**3*y)

rat[15] = (-u+u**3-u**4*v**2+v**2)/(-u**2*v+u**2*v**2*y+u**3*v*x-u**3*v**2*x*y
        +t*u**2-t*u**2*y*v-t*u**3*x+t*u**3*x*y*v+t*u**3*v**2-t*u**3*v**3*y
        -t*u**4*v**2*x+t*u**4*v**3*x*y+t*u*v**2-t*u*v**3*y-t*u**2*v**2*x
        +t*u**2*v**3*x*y)

rat[16] = (-u+u**3-u**4*v**2+v**2)/(-u**2*v+u**2*v**2*y+u**3*v*x-u**3*v**2*x*y
        +t*v*u-t*v**2*u*y-t*u**2*x*v+t*u**2*x*v**2*y+t*u**2-t*u**2*y*v
        -t*u**3*x+t*u**3*x*y*v+t*u**3*v-t*u**3*v**2*y-t*u**4*x*v
        +t*u**4*x*v**2*y+t*u**3*v**2-t*u**3*v**3*y-t*u**4*v**2*x
        +t*u**4*v**3*x*y+t*u*v**2-t*u*v**3*y-t*u**2*v**2*x
        +t*u**2*v**3*x*y)

rat[17] = (-u*v+u**3-u**4*v+u**3*v**3-v**4*u+v**3)/(-u**2*v**2+u**2*v**3*y
        +u**3*v**2*x-u**3*v**3*x*y+t*u**2*v-t*u**2*v**2*y-t*u**3*v*x
        +t*u**3*v**2*x*y+t*u**3*v**2-t*u**3*v**3*y-t*u**4*v**2*x
        +t*u**4*v**3*x*y+t*u*v**3-u*v**4*t*y-t*u**2*v**3*x
        +u**2*v**4*t*x*y)

rat[18] = (-u*v+u**3-u**4*v+u**3*v**3-v**4*u+v**3)/(t*u**2*v+t*u*v**2
        -t*u**3*v*x-t*u**2*v**2*x-t*u**2*v**2*y-t*u*v**3*y+t*u**3*v-u**2*v**2
        +u**2*v**4*t*x*y+t*u**3*v**2*x*y+t*u**2*v**3*x*y-t*u**3*v**2*y
        -t*u**2*v**3*x+t*u**2*v**3+t*u*v**3-t*u**3*v**3*x+u**2*v**3*y
        +u**3*v**2*x+t*u**3*v**2-u**3*v**3*x*y-t*u**3*v**3*y-t*u**4*v**2*x
        +t*u**4*v**3*x*y+u**3*v**4*t*x*y-t*u**4*x*v+t*u**4*x*v**2*y
        -u**2*v**4*t*y-u*v**4*t*y)

rat[19] = (-u**2*v+u**4-u**6+u**6*v-u**4*v**3+v**4*u**2-v**4+v**3)/(-u**3*v**2
        +u**3*v**3*y+u**4*v**2*x-u**4*v**3*x*y+t*u**2*v**2-t*u**2*v**3*y
        -t*u**3*v**2*x+t*u**3*v**3*x*y+u**4*t*v-u**4*t*v**2*y-u**5*t*x*v
        +u**5*t*x*v**2*y+t*u**4*v**2-t*u**4*v**3*y-u**5*t*x*v**2
        +u**5*t*x*v**3*y+t*u**2*v**3-u**2*v**4*t*y-t*u**3*v**3*x
        +u**3*v**4*t*x*y)


dop = {}

dop[1, 0, 0] = ( 96*t + (144*t**3 - 9*t)*Dt**2 + (16*t**4 -
t**2)*Dt**3 + (288*t**2 - 15)*Dt )

dop[2, 0, 0] = ( 32*t + (112*t**3 - 7*t)*Dt**2 + (16*t**4 -
t**2)*Dt**3 + (160*t**2 - 9)*Dt )

dop[3, 0, 0] = ( 10368*t**5 - 2016*t**3 + 192*t + (77760*t**7 -
23184*t**5 + 1992*t**3 - 30*t)*Dt**2 + (22464*t**8 - 7440*t**6 + 628*t**4
- 11*t**2)*Dt**3 + (1728*t**9 - 624*t**7 + 52*t**5 - t**3)*Dt**4 +
(72576*t**6 - 18720*t**4 + 1656*t**2 - 18)*Dt )

dop[4, 0, 0] = ( 1152*t**3 - 432*t**2 - 288*t + (960*t**6 + 368*t**5
- 414*t**4 - 54*t**3 + 11*t**2)*Dt**3 + (64*t**7 + 40*t**6 - 30*t**5 -
5*t**4 + t**3)*Dt**4 + (4032*t**5 + 552*t**4 - 1548*t**3 - 144*t**2 +
30*t)*Dt**2 + (4992*t**4 - 576*t**3 - 1620*t**2 - 84*t + 18)*Dt )

dop[5, 0, 0] = ( 384*t**3 + (576*t**5 - 9*t)*Dt**2 + (64*t**6 -
t**2)*Dt**3 + (1152*t**4 - 15)*Dt )

dop[6, 0, 0] = ( 4608*t**6 + 12672*t**5 + 16896*t**4 + 12672*t**3 +
4128*t**2 + 360*t + (13824*t**8 + 42048*t**7 + 56832*t**6 + 42816*t**5 +
15072*t**4 + 1068*t**3 - 264*t**2 - 108*t)*Dt**2 + (192*t**10 + 640*t**9
+ 880*t**8 + 656*t**7 + 244*t**6 + 16*t**5 - 7*t**4 - 3*t**3)*Dt**4 +
(3072*t**9 + 9792*t**8 + 13344*t**7 + 10016*t**6 + 3632*t**5 + 244*t**4
- 86*t**3 - 36*t**2)*Dt**3 + (18432*t**7 + 53376*t**6 + 71616*t**5 +
53952*t**4 + 18336*t**3 + 1416*t**2 - 180*t - 72)*Dt )

dop[7, 0, 0] = ( 288*t**3 + 48*t + (432*t**5 + 72*t**3 - 9*t)*Dt**2
+ (48*t**6 + 8*t**4 - t**2)*Dt**3 + (864*t**4 + 144*t**2 - 15)*Dt )

dop[8, 0, 0] = ( 24576*t**6 + 55296*t**5 + 53952*t**4 + 27072*t**3
+ 6792*t**2 + 720*t + (1024*t**10 + 2560*t**9 + 2688*t**8 + 1472*t**7 +
400*t**6 + 24*t**5 - 14*t**4 - 3*t**3)*Dt**4 + (16384*t**9 + 39936*t**8 +
41216*t**7 + 22144*t**6 + 5888*t**5 + 384*t**4 - 172*t**3 - 36*t**2)*Dt**3
+ (73728*t**8 + 175104*t**7 + 177504*t**6 + 93408*t**5 + 24324*t**4
+ 1800*t**3 - 528*t**2 - 108*t)*Dt**2 + (98304*t**7 + 227328*t**6 +
226176*t**5 + 116352*t**4 + 29712*t**3 + 2592*t**2 - 360*t - 72)*Dt )

dop[9, 0, 0] = ( 80640*t**7 - 1344*t**3 + 96*t + (2688*t**11 +
2048*t**9 - 594*t**7 + 45*t**5 - t**3)*Dt**4 + (217728*t**9 + 73728*t**7
- 21780*t**5 + 1590*t**3 - 30*t)*Dt**2 + (306432*t**8 + 49152*t**6 -
16668*t**4 + 1194*t**2 - 18)*Dt + (45696*t**10 + 24576*t**8 - 7082*t**6 +
527*t**4 - 11*t**2)*Dt**3 )

dop[10, 0, 0] = ( 23040*t**8 + 45696*t**7 - 21888*t**6 - 28800*t**5
- 5472*t**4 + 5400*t**3 + 2808*t**2 + 384*t + (69120*t**10 + 133056*t**9 -
39360*t**8 - 106368*t**7 - 88800*t**6 - 2028*t**5 + 16788*t**4 + 3696*t**3
- 156*t**2 - 60*t)*Dt**2 + (960*t**12 + 1792*t**11 - 208*t**10 - 1936*t**9
- 2508*t**8 - 408*t**7 + 357*t**6 + 95*t**5 - 5*t**4 - 2*t**3)*Dt**4 +
(92160*t**9 + 180096*t**8 - 69696*t**7 - 126336*t**6 - 66912*t**5 +
10968*t**4 + 16716*t**3 + 3132*t**2 - 96*t - 36)*Dt + (15360*t**11 +
29120*t**10 - 5984*t**9 - 26944*t**8 - 29392*t**7 - 3236*t**6 + 4706*t**5
+ 1154*t**4 - 56*t**3 - 22*t**2)*Dt**3 )

dop[11, 0, 0] = ( 288*t**3 + 48*t + (432*t**5 + 72*t**3 - 9*t)*Dt**2
+ (48*t**6 + 8*t**4 - t**2)*Dt**3 + (864*t**4 + 144*t**2 - 15)*Dt )

dop[12, 0, 0] = ( 24576*t**6 + 55296*t**5 + 53952*t**4 +
27072*t**3 + 6792*t**2 + 720*t + (73728*t**8 + 175104*t**7 + 177504*t**6
+ 93408*t**5 + 24324*t**4 + 1800*t**3 - 528*t**2 - 108*t)*Dt**2 +
(1024*t**10 + 2560*t**9 + 2688*t**8 + 1472*t**7 + 400*t**6 + 24*t**5 -
14*t**4 - 3*t**3)*Dt**4 + (98304*t**7 + 227328*t**6 + 226176*t**5 +
116352*t**4 + 29712*t**3 + 2592*t**2 - 360*t - 72)*Dt + (16384*t**9 +
39936*t**8 + 41216*t**7 + 22144*t**6 + 5888*t**5 + 384*t**4 - 172*t**3 -
36*t**2)*Dt**3 )

dop[13, 0, 0] = ( 80640*t**7 - 1344*t**3 + 96*t + (217728*t**9
+ 73728*t**7 - 21780*t**5 + 1590*t**3 - 30*t)*Dt**2 + (2688*t**11 +
2048*t**9 - 594*t**7 + 45*t**5 - t**3)*Dt**4 + (45696*t**10 + 24576*t**8
- 7082*t**6 + 527*t**4 - 11*t**2)*Dt**3 + (306432*t**8 + 49152*t**6 -
16668*t**4 + 1194*t**2 - 18)*Dt )

dop[14, 0, 0] = ( 23040*t**8 + 45696*t**7 - 21888*t**6 - 28800*t**5
- 5472*t**4 + 5400*t**3 + 2808*t**2 + 384*t + (960*t**12 + 1792*t**11 -
208*t**10 - 1936*t**9 - 2508*t**8 - 408*t**7 + 357*t**6 + 95*t**5 -
5*t**4 - 2*t**3)*Dt**4 + (69120*t**10 + 133056*t**9 - 39360*t**8 -
106368*t**7 - 88800*t**6 - 2028*t**5 + 16788*t**4 + 3696*t**3 - 156*t**2
- 60*t)*Dt**2 + (15360*t**11 + 29120*t**10 - 5984*t**9 - 26944*t**8 -
29392*t**7 - 3236*t**6 + 4706*t**5 + 1154*t**4 - 56*t**3 - 22*t**2)*Dt**3
+ (92160*t**9 + 180096*t**8 - 69696*t**7 - 126336*t**6 - 66912*t**5 +
10968*t**4 + 16716*t**3 + 3132*t**2 - 96*t - 36)*Dt )

dop[15, 0, 0] = ( 384*t**3 + (64*t**6 - t**2)*Dt**3 + (576*t**5 -
9*t)*Dt**2 + (1152*t**4 - 15)*Dt )

dop[16, 0, 0] = ( 4608*t**6 + 12672*t**5 + 16896*t**4 + 12672*t**3 +
4128*t**2 + 360*t + (13824*t**8 + 42048*t**7 + 56832*t**6 + 42816*t**5 +
15072*t**4 + 1068*t**3 - 264*t**2 - 108*t)*Dt**2 + (192*t**10 + 640*t**9
+ 880*t**8 + 656*t**7 + 244*t**6 + 16*t**5 - 7*t**4 - 3*t**3)*Dt**4 +
(18432*t**7 + 53376*t**6 + 71616*t**5 + 53952*t**4 + 18336*t**3 +
1416*t**2 - 180*t - 72)*Dt + (3072*t**9 + 9792*t**8 + 13344*t**7 +
10016*t**6 + 3632*t**5 + 244*t**4 - 86*t**3 - 36*t**2)*Dt**3 )

dop[17, 0, 0] = ( 162*t**2 + (27*t**5 - t**2)*Dt**3 + (243*t**4 -
12*t)*Dt**2 + (486*t**3 - 28)*Dt )

dop[18, 0, 0] = ( 864*t**2 + 360*t + (2592*t**4 + 1404*t**3
+ 24*t**2 - 50*t)*Dt**2 + (3456*t**3 + 1656*t**2 + 12*t - 40)*Dt +
(36*t**6 + 24*t**5 + t**4 - t**3)*Dt**4 + (576*t**5 + 348*t**4 + 10*t**3 -
14*t**2)*Dt**3 )

dop[19, 0, 0] = ( 384*t + (1152*t**3 - 87*t)*Dt**2 + (16*t**5 -
t**3)*Dt**4 + (1536*t**2 - 105)*Dt + (256*t**4 - 18*t**2)*Dt**3 )

dop[1, 1, 0] = ( 768*t**2 + 864*t + 72 + (3456*t**4 + 3120*t**3 -
12*t**2 - 108*t)*Dt**2 + (896*t**5 + 736*t**4 - 32*t**3 - 36*t**2)*Dt**3 +
(64*t**6 + 48*t**5 - 4*t**4 - 3*t**3)*Dt**4 + (3840*t**3 + 3840*t**2 +
144*t - 72)*Dt )

dop[2, 1, 0] = ( 96*t + (208*t**4 - 9*t**2)*Dt**3 + (16*t**5 -
t**3)*Dt**4 + (720*t**3 - 18*t)*Dt**2 + (672*t**2 - 6)*Dt )

dop[3, 1, 0] = ( 1440*t**3 - 240*t + (8496*t**5 - 1752*t**3 +
18*t)*Dt**2 + (2160*t**6 - 512*t**4 + 9*t**2)*Dt**3 + (144*t**7 -
40*t**5 + t**3)*Dt**4 + (8928*t**4 - 1632*t**2 + 6)*Dt )

dop[4, 1, 0] = ( 4608*t**4 + 6720*t**3 + 792*t**2 - 360*t - 12 +
(16128*t**6 + 27360*t**5 + 9564*t**4 - 1440*t**3 - 234*t**2 + 24*t)*Dt**2
+ (256*t**8 + 512*t**7 + 244*t**6 - 23*t**5 - 10*t**4 + t**3)*Dt**4 +
(19968*t**5 + 31296*t**4 + 7824*t**3 - 1704*t**2 - 132*t + 12)*Dt +
(3840*t**7 + 7072*t**6 + 2992*t**5 - 348*t**4 - 98*t**3 + 10*t**2)*Dt**3 )

dop[5, 1, 0] = ( 384*t**3 + 32*t + (64*t**6 - t**2)*Dt**3 +
(576*t**5 + 16*t**3 - 7*t)*Dt**2 + (1152*t**4 + 64*t**2 - 9)*Dt )

dop[6, 1, 0] = ( 9216*t**7 + 23040*t**6 + 26496*t**5 + 18720*t**4 +
8352*t**3 + 2160*t**2 + 288*t + 18 + (384*t**11 + 1280*t**10 + 1664*t**9
+ 1136*t**8 + 456*t**7 + 88*t**6 - 4*t**5 - 5*t**4 - t**3)*Dt**4 +
(27648*t**9 + 80640*t**8 + 98880*t**7 + 69360*t**6 + 29712*t**5 +
6888*t**4 + 456*t**3 - 105*t**2 - 30*t)*Dt**2 + (6144*t**10 + 19200*t**9
+ 24256*t**8 + 16816*t**7 + 6992*t**6 + 1496*t**5 + 28*t**4 - 49*t**3 -
11*t**2)*Dt**3 + (36864*t**8 + 99840*t**7 + 118656*t**6 + 83808*t**5 +
36768*t**4 + 9072*t**3 + 936*t**2 - 18*t - 18)*Dt )

dop[7, 1, 0] = ( 288*t**3 + 64*t + (48*t**6 + 8*t**4 - t**2)*Dt**3 +
(432*t**5 + 80*t**3 - 7*t)*Dt**2 + (864*t**4 + 176*t**2 - 9)*Dt )

dop[8, 1, 0] = ( 73728*t**7 + 156672*t**6 + 157248*t**5 + 93312*t**4
+ 34056*t**3 + 7416*t**2 + 864*t + 36 + (3072*t**11 + 7680*t**10 +
8320*t**9 + 5056*t**8 + 1776*t**7 + 296*t**6 - 14*t**5 - 15*t**4 -
2*t**3)*Dt**4 + (221184*t**9 + 511488*t**8 + 534816*t**7 + 323136*t**6
+ 116868*t**5 + 23052*t**4 + 1296*t**3 - 366*t**2 - 60*t)*Dt**2 +
(49152*t**10 + 118272*t**9 + 125952*t**8 + 76416*t**7 + 27296*t**6
+ 5000*t**5 + 60*t**4 - 154*t**3 - 22*t**2)*Dt**3 + (294912*t**8 +
654336*t**7 + 670848*t**6 + 402432*t**5 + 146640*t**4 + 30624*t**3 +
2760*t**2 - 132*t - 36)*Dt )

dop[9, 1, 0] = ( 74880*t**7 - 7392*t**5 - 2784*t**3 + 288*t +
(42432*t**10 + 19120*t**8 - 8929*t**6 + 865*t**4 - 18*t**2)*Dt**3 +
(2496*t**11 + 1696*t**9 - 723*t**7 + 75*t**5 - 2*t**3)*Dt**4 +
(202176*t**9 + 49968*t**7 - 28938*t**5 + 2634*t**3 - 36*t)*Dt**2 +
(284544*t**8 + 18528*t**6 - 24438*t**4 + 2166*t**2 - 12)*Dt )

dop[10, 1, 0] = ( 19200*t**8 + 112128*t**7 - 13440*t**6 -
63360*t**5 - 41616*t**4 - 7056*t**3 + 960*t**2 + 300*t + 12 + (640*t**12
+ 3328*t**11 + 768*t**10 - 1808*t**9 - 4584*t**8 - 1944*t**7 - 4*t**6 +
83*t**5 + 3*t**4 - t**3)*Dt**4 + (51840*t**10 + 289536*t**9 + 22848*t**8 -
146496*t**7 - 221688*t**6 - 73512*t**5 + 288*t**4 + 2406*t**3 + 66*t**2
- 24*t)*Dt**2 + (72960*t**9 + 417792*t**8 - 6528*t**7 - 219840*t**6 -
229104*t**5 - 60768*t**4 + 1776*t**3 + 1860*t**2 + 48*t - 12)*Dt +
(10880*t**11 + 58880*t**10 + 9536*t**9 - 29984*t**8 - 60984*t**7 -
23328*t**6 - 56*t**5 + 874*t**4 + 28*t**3 - 10*t**2)*Dt**3 )

dop[11, 1, 0] = ( 288*t**3 + 64*t + (432*t**5 + 80*t**3 - 7*t)*Dt**2
+ (48*t**6 + 8*t**4 - t**2)*Dt**3 + (864*t**4 + 176*t**2 - 9)*Dt )

dop[12, 1, 0] = ( 73728*t**7 + 156672*t**6 + 157248*t**5 +
93312*t**4 + 34056*t**3 + 7416*t**2 + 864*t + 36 + (221184*t**9 +
511488*t**8 + 534816*t**7 + 323136*t**6 + 116868*t**5 + 23052*t**4 +
1296*t**3 - 366*t**2 - 60*t)*Dt**2 + (3072*t**11 + 7680*t**10 + 8320*t**9
+ 5056*t**8 + 1776*t**7 + 296*t**6 - 14*t**5 - 15*t**4 - 2*t**3)*Dt**4 +
(49152*t**10 + 118272*t**9 + 125952*t**8 + 76416*t**7 + 27296*t**6
+ 5000*t**5 + 60*t**4 - 154*t**3 - 22*t**2)*Dt**3 + (294912*t**8 +
654336*t**7 + 670848*t**6 + 402432*t**5 + 146640*t**4 + 30624*t**3 +
2760*t**2 - 132*t - 36)*Dt )

dop[13, 1, 0] = ( 74880*t**7 - 7392*t**5 - 2784*t**3 + 288*t
+ (2496*t**11 + 1696*t**9 - 723*t**7 + 75*t**5 - 2*t**3)*Dt**4 +
(202176*t**9 + 49968*t**7 - 28938*t**5 + 2634*t**3 - 36*t)*Dt**2 +
(42432*t**10 + 19120*t**8 - 8929*t**6 + 865*t**4 - 18*t**2)*Dt**3 +
(284544*t**8 + 18528*t**6 - 24438*t**4 + 2166*t**2 - 12)*Dt )

dop[14, 1, 0] = ( 19200*t**8 + 112128*t**7 - 13440*t**6 - 63360*t**5
- 41616*t**4 - 7056*t**3 + 960*t**2 + 300*t + 12 + (51840*t**10 +
289536*t**9 + 22848*t**8 - 146496*t**7 - 221688*t**6 - 73512*t**5 +
288*t**4 + 2406*t**3 + 66*t**2 - 24*t)*Dt**2 + (640*t**12 + 3328*t**11 +
768*t**10 - 1808*t**9 - 4584*t**8 - 1944*t**7 - 4*t**6 + 83*t**5 + 3*t**4
- t**3)*Dt**4 + (72960*t**9 + 417792*t**8 - 6528*t**7 - 219840*t**6 -
229104*t**5 - 60768*t**4 + 1776*t**3 + 1860*t**2 + 48*t - 12)*Dt +
(10880*t**11 + 58880*t**10 + 9536*t**9 - 29984*t**8 - 60984*t**7 -
23328*t**6 - 56*t**5 + 874*t**4 + 28*t**3 - 10*t**2)*Dt**3 )

dop[15, 1, 0] = ( 384*t**3 + 32*t + (576*t**5 + 16*t**3 - 7*t)*Dt**2
+ (64*t**6 - t**2)*Dt**3 + (1152*t**4 + 64*t**2 - 9)*Dt )

dop[16, 1, 0] = ( 9216*t**7 + 23040*t**6 + 26496*t**5 + 18720*t**4 +
8352*t**3 + 2160*t**2 + 288*t + 18 + (384*t**11 + 1280*t**10 + 1664*t**9
+ 1136*t**8 + 456*t**7 + 88*t**6 - 4*t**5 - 5*t**4 - t**3)*Dt**4 +
(27648*t**9 + 80640*t**8 + 98880*t**7 + 69360*t**6 + 29712*t**5 +
6888*t**4 + 456*t**3 - 105*t**2 - 30*t)*Dt**2 + (6144*t**10 + 19200*t**9
+ 24256*t**8 + 16816*t**7 + 6992*t**6 + 1496*t**5 + 28*t**4 - 49*t**3 -
11*t**2)*Dt**3 + (36864*t**8 + 99840*t**7 + 118656*t**6 + 83808*t**5 +
36768*t**4 + 9072*t**3 + 936*t**2 - 18*t - 18)*Dt )

dop[17, 1, 0] = ( 524880*t**5 + 3324240*t**4 - 388800*t**3 -
90720*t**2 + 7200 + (3936600*t**7 + 24494400*t**6 + 1409400*t**5 -
3645000*t**4 + 152280*t**3 + 24480*t**2 + 25200*t - 3600)*Dt**2 +
(3149280*t**6 + 19770480*t**5 - 544320*t**4 - 1704240*t**3 + 51840*t**2 +
43200*t)*Dt + (729*t**11 + 4374*t**10 + 1539*t**9 - 1863*t**8 + 108*t**7
- 57*t**6 + 68*t**5 - 10*t**4)*Dt**6 + (1749600*t**8 + 10789200*t**7 +
1490400*t**6 - 2315520*t**5 + 113400*t**4 - 13560*t**3 + 33360*t**2 -
4800*t)*Dt**3 + (328050*t**9 + 2004750*t**8 + 429300*t**7 - 567540*t**6 +
30510*t**5 - 8910*t**4 + 12420*t**3 - 1800*t**2)*Dt**4 + (26244*t**10 +
158922*t**9 + 45360*t**8 - 56214*t**7 + 3186*t**6 - 1338*t**5 + 1644*t**4
- 240*t**3)*Dt**5 )

dop[18, 1, 0] = ( 288*t + 24 + (864*t**3 + 180*t**2 - 36*t)*Dt**2 +
(1152*t**2 + 168*t - 24)*Dt + (12*t**5 + 4*t**4 - t**3)*Dt**4 + (192*t**4
+ 52*t**3 - 12*t**2)*Dt**3 )

dop[19, 1, 0] = ( 384*t**2 - 176*t - 60 + (1728*t**4 - 856*t**3 -
176*t**2 + 60*t)*Dt**2 + (448*t**5 - 224*t**4 - 36*t**3 + 15*t**2)*Dt**3 +
(1920*t**3 - 928*t**2 - 244*t + 60)*Dt + (32*t**6 - 16*t**5 - 2*t**4 +
t**3)*Dt**4 )

dop[1, 0, 1] = ( 768*t**2 + 864*t + 72 + (3456*t**4 + 3120*t**3 -
12*t**2 - 108*t)*Dt**2 + (64*t**6 + 48*t**5 - 4*t**4 - 3*t**3)*Dt**4 +
(896*t**5 + 736*t**4 - 32*t**3 - 36*t**2)*Dt**3 + (3840*t**3 + 3840*t**2 +
144*t - 72)*Dt )

dop[2, 0, 1] = ( 96*t + (16*t**5 - t**3)*Dt**4 + (720*t**3 -
18*t)*Dt**2 + (208*t**4 - 9*t**2)*Dt**3 + (672*t**2 - 6)*Dt )

dop[3, 0, 1] = ( 62208*t**6 - 51840*t**5 - 3456*t**4 + 6048*t**3
+ 576*t**2 - 192*t - 12 + (466560*t**8 - 295488*t**7 - 107136*t**6 +
54864*t**5 + 9936*t**4 - 2064*t**3 - 138*t**2 + 24*t)*Dt**2 + (10368*t**10
- 5184*t**9 - 3744*t**8 + 1584*t**7 + 312*t**6 - 76*t**5 - 6*t**4 +
t**3)*Dt**4 + (134784*t**9 - 76032*t**8 - 40032*t**7 + 18144*t**6
+ 3480*t**5 - 784*t**4 - 58*t**3 + 10*t**2)*Dt**3 + (435456*t**7 -
311040*t**6 - 67392*t**5 + 44928*t**4 + 7056*t**3 - 1440*t**2 - 84*t +
12)*Dt )

dop[4, 0, 1] = ( 4608*t**4 + 6720*t**3 + 792*t**2 - 360*t - 12 +
(256*t**8 + 512*t**7 + 244*t**6 - 23*t**5 - 10*t**4 + t**3)*Dt**4 +
(3840*t**7 + 7072*t**6 + 2992*t**5 - 348*t**4 - 98*t**3 + 10*t**2)*Dt**3 +
(16128*t**6 + 27360*t**5 + 9564*t**4 - 1440*t**3 - 234*t**2 + 24*t)*Dt**2
+ (19968*t**5 + 31296*t**4 + 7824*t**3 - 1704*t**2 - 132*t + 12)*Dt )

dop[5, 0, 1] = ( 4282122240*t**12 - 1817763840*t**11 -
1140940800*t**10 + 1986785280*t**9 - 901693440*t**8 - 292700160*t**7 -
19359360*t**6 + 11456640*t**5 - 3709440*t**4 - 789120*t**3 - 129600*t**2
- 25920*t + (14273740800*t**15 - 9024921600*t**14 - 5929697280*t**13 +
8921395200*t**12 - 3360890880*t**11 - 1959208320*t**10 + 1442588160*t**9 -
69008640*t**8 - 36064320*t**7 + 2130240*t**6 + 2550480*t**5 - 999480*t**4
+ 77520*t**3 + 11220*t**2 - 6960*t + 600)*Dt**3 + (32115916800*t**14
- 18081792000*t**13 - 10281000960*t**12 + 17049600000*t**11 -
7303127040*t**10 - 3088967040*t**9 + 1697685120*t**8 + 62205120*t**7 -
47547360*t**6 - 6363360*t**5 + 7730280*t**4 - 1068840*t**3 + 11160*t**2
+ 20880*t - 3600)*Dt**2 + (25692733440*t**13 - 12686008320*t**12 -
6948864000*t**11 + 12300595200*t**10 - 5672263680*t**9 - 1833494400*t**8
+ 424224000*t**7 + 112593600*t**6 - 22273920*t**5 - 6216480*t**4 +
2477520*t**3 + 88920*t**2 + 14400*t + 3600)*Dt + (5947392*t**18
- 4996096*t**17 - 5799936*t**16 + 7175680*t**15 - 1855488*t**14
- 1865744*t**13 + 1755840*t**12 - 352056*t**11 - 24492*t**10 +
34916*t**9 - 24099*t**8 + 3365*t**7 + 813*t**6 - 71*t**5 - 30*t**4 +
6*t**3)*Dt**6 + (214106112*t**17 - 165030912*t**16 - 159074304*t**15 +
205146624*t**14 - 58622976*t**13 - 53464464*t**12 + 47860896*t**11 -
7315656*t**10 - 945408*t**9 + 608508*t**8 - 353082*t**7 + 29121*t**6 +
13620*t**5 - 897*t**4 - 588*t**3 + 102*t**2)*Dt**5 + (2676326400*t**16 -
1877529600*t**15 - 1489044480*t**14 + 2049753600*t**13 - 667852800*t**12
- 507937920*t**11 + 424285200*t**10 - 44238480*t**9 - 9819600*t**8 +
2831760*t**7 - 940380*t**6 - 111360*t**5 + 62760*t**4 - 1140*t**3 -
3480*t**2 + 480*t)*Dt**4 )

dop[6, 0, 1] = ( 389372313600*t**16 + 1245045841920*t**15 +
1943895490560*t**14 + 1509047193600*t**13 + 350940026880*t**12
- 168346552320*t**11 - 120867586560*t**10 - 92523444480*t**9 -
67971196800*t**8 - 28993839840*t**7 - 6897987360*t**6 - 820038240*t**5
- 116471520*t**4 - 22733280*t**3 - 1717920*t**2 - 108000*t + 7200 +
(2920292352000*t**18 + 9458701516800*t**17 + 13991759493120*t**16
+ 9639228303360*t**15 + 584988042240*t**14 - 2398744857600*t**13
- 862535784960*t**12 - 344250950400*t**11 - 269234562240*t**10 -
50490673920*t**9 + 22610929320*t**8 + 13633769160*t**7 + 2022055560*t**6 +
128875320*t**5 + 22749480*t**4 + 3353400*t**3 - 151560*t**2 - 87480*t -
3600)*Dt**2 + (540794880*t**22 + 1796373504*t**21 + 2015610624*t**20 -
32511744*t**19 - 2329178496*t**18 - 2072294528*t**17 - 353095808*t**16 +
258456064*t**15 + 100822328*t**14 + 43008412*t**13 + 25357497*t**12 +
7681443*t**11 - 190170*t**10 - 719070*t**9 - 125132*t**8 + 5980*t**7 +
4282*t**6 - 70*t**5 - 5*t**4 + 9*t**3)*Dt**6 + (2336233881600*t**17 +
7518618132480*t**16 + 11492808560640*t**15 + 8569896284160*t**14 +
1550154424320*t**13 - 1284030627840*t**12 - 682313195520*t**11
- 435746672640*t**10 - 314590590720*t**9 - 108319485600*t**8 -
12548195280*t**7 + 2718414000*t**6 + 540216000*t**5 + 54669600*t**4 +
19036080*t**3 + 2752560*t**2 + 303840*t - 4320)*Dt + (1297907712000*t**19
+ 4230724608000*t**18 + 5980650024960*t**17 + 3580509265920*t**16
- 659289077760*t**15 - 1646255585280*t**14 - 427062881280*t**13
- 34502730240*t**12 - 57420168960*t**11 + 14633268000*t**10 +
27013582800*t**9 + 10485978720*t**8 + 1197567480*t**7 - 56415720*t**6
- 19419120*t**5 - 758400*t**4 - 353640*t**3 - 105000*t**2 -
5040*t + 720)*Dt**3 + (243357696000*t**20 + 798296601600*t**19 +
1063343324160*t**18 + 493403420160*t**17 - 360520339200*t**16
- 464543320320*t**15 - 97028487360*t**14 + 23216326560*t**13 +
3352126800*t**12 + 9216690540*t**11 + 7960498800*t**10 + 2688900660*t**9
+ 201865650*t**8 - 68151090*t**7 - 15620310*t**6 - 556890*t**5 +
38670*t**4 - 33450*t**3 - 1770*t**2 + 630*t)*Dt**4 + (19468615680*t**21
+ 64266587136*t**20 + 79351543296*t**19 + 21273546240*t**18
- 53358337536*t**17 - 53711365632*t**16 - 9868011264*t**15 +
5041815936*t**14 + 1714286688*t**13 + 1189260996*t**12 + 817499490*t**11 +
257312238*t**10 + 7582266*t**9 - 13620954*t**8 - 2697690*t**7 - 5526*t**6
+ 46110*t**5 - 3282*t**4 - 192*t**3 + 144*t**2)*Dt**5 )

dop[7, 0, 1] = ( 14929920*t**11 + 1492992*t**10 - 14618880*t**9 +
8933760*t**8 - 1593216*t**7 - 116352*t**6 - 641232*t**5 + 29736*t**4 -
27528*t**3 - 6156*t**2 - 864*t - 180 + (124416000*t**14 - 71165952*t**13
- 84882816*t**12 + 121993920*t**11 - 69554304*t**10 + 22439040*t**9 -
7235832*t**8 + 2240580*t**7 - 403760*t**6 + 145350*t**5 + 222*t**4 -
5138*t**3 + 90*t**2)*Dt**3 + (82944*t**17 - 103680*t**16 - 20736*t**15
+ 85632*t**14 - 71040*t**13 + 36928*t**12 - 12464*t**11 + 1944*t**10 +
652*t**9 - 175*t**8 + 14*t**7 - 24*t**6 + 5*t**5)*Dt**6 + (223948800*t**13
- 77635584*t**12 - 178858368*t**11 + 199471680*t**10 - 93797568*t**9 +
23364288*t**8 - 9696888*t**7 + 3150876*t**6 - 775308*t**5 + 197394*t**4 +
9012*t**3 - 954*t**2 - 180*t)*Dt**2 + (134369280*t**12 - 16422912*t**11
- 120828672*t**10 + 102591360*t**9 - 35403264*t**8 + 5038848*t**7 -
4970736*t**6 + 1219176*t**5 - 369408*t**4 + 28884*t**3 + 7020*t**2 +
1044*t + 180)*Dt + (2612736*t**16 - 2674944*t**15 - 1064448*t**14
+ 2745216*t**13 - 2035584*t**12 + 916128*t**11 - 291024*t**10 +
59816*t**9 + 5162*t**8 - 485*t**7 + 83*t**6 - 481*t**5 + 75*t**4)*Dt**5
+ (27993600*t**15 - 22332672*t**14 - 15441408*t**13 + 28935360*t**12 -
19027584*t**11 + 7350336*t**10 - 2262480*t**9 + 587892*t**8 - 44326*t**7 +
24173*t**6 - 372*t**5 - 2839*t**4 + 270*t**3)*Dt**4 )

dop[8, 0, 1] = ( (67947724800*t**15 + 243479347200*t**14
+ 435856343040*t**13 + 577888911360*t**12 + 637509980160*t**11
+ 545758110720*t**10 + 333518169600*t**9 + 139739218560*t**8 +
39300128640*t**7 + 7229682720*t**6 + 825151680*t**5 + 48402720*t**4 -
1175760*t**3 - 658080*t**2 - 76320*t - 2880)*Dt**2 + (37748736*t**19 +
154140672*t**18 + 298549248*t**17 + 390463488*t**16 + 404615168*t**15 +
337235968*t**14 + 210322944*t**13 + 89454080*t**12 + 22268096*t**11 +
1347712*t**10 - 986752*t**9 - 306848*t**8 - 25804*t**7 + 3228*t**6
+ 750*t**5 + 55*t**4 + 4*t**3)*Dt**6 + (45298483200*t**16 +
167981875200*t**15 + 306838241280*t**14 + 404874362880*t**13 +
440421150720*t**12 + 376757015040*t**11 + 233213844480*t**10 +
99091859520*t**9 + 27646967040*t**8 + 4632037200*t**7 + 318341280*t**6
- 36330000*t**5 - 11176920*t**4 - 1298880*t**3 - 83280*t**2 - 840*t +
240)*Dt**3 + (27179089920*t**14 + 93994352640*t**13 + 164844011520*t**12
+ 219865743360*t**11 + 245662801920*t**10 + 209554099200*t**9 +
125659607040*t**8 + 51457115520*t**7 + 14347837440*t**6 + 2757581280*t**5
+ 379494720*t**4 + 39949920*t**3 + 3298320*t**2 + 198720*t +
5760)*Dt + (1132462080*t**18 + 4482662400*t**17 + 8517058560*t**16
+ 11162959872*t**15 + 11767676928*t**14 + 9931597824*t**13 +
6208785408*t**12 + 2656495872*t**11 + 698257152*t**10 + 75724512*t**9 -
13941648*t**8 - 6113352*t**7 - 746844*t**6 + 696*t**5 + 7524*t**4 +
714*t**3 + 60*t**2)*Dt**5 + (11324620800*t**17 + 43411046400*t**16
+ 80885514240*t**15 + 106323148800*t**14 + 113915335680*t**13
+ 96981742080*t**12 + 60480336000*t**11 + 25870933440*t**10 +
7057800480*t**9 + 1008626160*t**8 - 13470720*t**7 - 32812800*t**6 -
5518920*t**5 - 354960*t**4 + 900*t**3 + 1950*t**2 + 240*t)*Dt**4 )

dop[9, 0, 1] = ( - 6480 + 8640*t + 562320*t**2 + 142754744160*t**11
- 35125200*t**4 - 3504960*t**3 - 25274630983680*t**16 + 107834400*t**5 +
(117814984704000*t**23 + 957701814681600*t**22 - 1127742734499840*t**21 -
1370648690933760*t**20 + 477758169907200*t**19 + 76368977902080*t**18
+ 91159374481920*t**17 + 35859151664640*t**16 - 51259001587200*t**15
- 3095185857120*t**14 + 3284382409920*t**13 - 499838718240*t**12
+ 582019841520*t**11 + 97371344160*t**10 - 104414498280*t**9 -
6717435120*t**8 + 7194247920*t**7 + 294734520*t**6 - 257064480*t**5 -
10412280*t**4 + 4281120*t**3 + 86760*t**2 - 29880*t + 360)*Dt**2 +
(21817589760*t**27 + 176489238528*t**26 - 35953238016*t**25 -
225247758336*t**24 - 71349571584*t**23 - 182412219456*t**22 -
16404090240*t**21 + 171120067392*t**20 - 49323744*t**19 -
44090285556*t**18 + 2391533964*t**17 + 4207901220*t**16 -
384232764*t**15 + 47941638*t**14 + 4438053*t**13 - 41811448*t**12
+ 3774511*t**11 + 3575084*t**10 - 390090*t**9 - 145488*t**8 +
17130*t**7 + 3074*t**6 - 359*t**5 - 28*t**4 + 3*t**3)*Dt**6 +
(785433231360*t**26 + 6361379131392*t**25 - 2749302964224*t**24 -
8810298243072*t**23 - 1970033909760*t**22 - 3596321581632*t**21
+ 471014618112*t**20 + 3844937856960*t**19 - 410565800064*t**18
- 878448401700*t**17 + 101523180504*t**16 + 59737369452*t**15 -
6452989152*t**14 + 4505168118*t**13 - 641712558*t**12 - 1006994244*t**11
+ 132260508*t**10 + 71074932*t**9 - 9423600*t**8 - 2625432*t**7 +
339060*t**6 + 52122*t**5 - 6234*t**4 - 432*t**3 + 48*t**2)*Dt**5 +
(9817915392000*t**25 + 79614320947200*t**24 - 53395331051520*t**23 -
115267690690560*t**22 - 9923820779520*t**21 - 17419855472640*t**20 +
11467582344000*t**19 + 26564792178240*t**18 - 6686518161600*t**17
- 5241093309360*t**16 + 1023181263300*t**15 + 178395909180*t**14
- 8117008560*t**13 + 51871555560*t**12 - 11916505860*t**11 -
7184939550*t**10 + 1370012310*t**9 + 424970340*t**8 - 74005200*t**7 -
14121300*t**6 + 2119560*t**5 + 254340*t**4 - 31920*t**3 - 1770*t**2
+ 210*t)*Dt**4 + (94251987763200*t**22 + 767093437071360*t**21 -
1109114374717440*t**20 - 1035021700546560*t**19 + 712224547799040*t**18
+ 12941904314880*t**17 + 19876874342400*t**16 - 6120756403200*t**15
- 23576905681920*t**14 + 2782408119840*t**13 + 820264345920*t**12
+ 12354932160*t**11 + 335583794880*t**10 - 53809161840*t**9 -
45793987920*t**8 + 5324557680*t**7 + 2710640880*t**6 - 189696240*t**5
- 83794320*t**4 + 1962000*t**3 + 1046160*t**2 - 40320*t - 7200)*Dt +
(52362215424000*t**24 + 425127481344000*t**23 - 390752475217920*t**22 -
622231263313920*t**21 + 62217307422720*t**20 - 555553175040*t**19 +
60676338216960*t**18 + 62891854951680*t**17 - 32068802373120*t**16
- 10094950101600*t**15 + 3331121388480*t**14 - 141666162480*t**13
+ 190287261120*t**12 + 162609493200*t**11 - 61758258000*t**10 -
16576522440*t**9 + 5252061960*t**8 + 831399480*t**7 - 226666200*t**6
- 25139160*t**5 + 5006760*t**4 + 378360*t**3 - 54840*t**2 -
1680*t + 240)*Dt**3 - 11710029600*t**8 - 220685836124160*t**19 +
143152012800*t**14 - 2227663779840*t**13 + 520516045440*t**12 +
19329029280*t**10 + 10364775840*t**9 - 1719303840*t**7 + 1003775760*t**6
+ 15708664627200*t**21 + 128004237066240*t**20 - 155744725155840*t**18 +
183151373598720*t**17 - 5273122429440*t**15 )

dop[10, 0, 1] = ( 27648 + 67392*t + 13107429743616*t**11 +
620174880*t**4 + 17416800*t**3 - 89408181888*t**8 - 3117024*t**2
+ 3525651072*t**5 - 99851923980288*t**14 + 5527550016*t**6 +
6936330240000*t**21 - 34871574528000*t**20 - 87726788444160*t**19 +
114805845393408*t**18 + 102447992438784*t**16 - 130395859697664*t**15
+ 380687634432*t**13 + 26523749855232*t**12 + 2777655439872*t**10
+ 101825316864*t**9 - 17013261312*t**7 + 304142079688704*t**17 +
(9633792000*t**27 - 33787084800*t**26 - 119487528960*t**25 +
25109774336*t**24 + 358429102080*t**23 + 408355031040*t**22 -
38041862144*t**21 - 491979262976*t**20 - 441838339072*t**19 -
106531263488*t**18 + 71579604096*t**17 + 57700640640*t**16 +
13971845040*t**15 - 199562440*t**14 - 681638360*t**13 - 89585416*t**12 +
10641872*t**11 + 5194714*t**10 + 623600*t**9 - 114411*t**8 - 36000*t**7 +
91*t**6 + 595*t**5 + 14*t**4 - 3*t**3)*Dt**6 + (23121100800000*t**24 -
98663792640000*t**23 - 289604110909440*t**22 + 216503666737152*t**21 +
931933960863744*t**20 + 640470519939072*t**19 - 283442923585536*t**18 -
665081310474240*t**17 - 349561410502656*t**16 + 18370714641408*t**15 +
103411977345024*t**14 + 48934114835712*t**13 + 9534037239552*t**12
+ 302579400000*t**11 - 189777902400*t**10 - 18853874880*t**9 +
12817057056*t**8 + 5443751856*t**7 + 588501648*t**6 - 75660696*t**5
- 16169376*t**4 + 132408*t**3 + 110496*t**2 - 408*t - 240)*Dt**3 +
(346816512000*t**26 - 1304208998400*t**25 - 4315747123200*t**24 +
1680668639232*t**23 + 13260854059008*t**22 + 12829341941760*t**21 -
2615915182080*t**20 - 14901856800768*t**19 - 11639540711424*t**18 -
2030771578368*t**17 + 2211992136192*t**16 + 1513245696576*t**15
+ 350425089120*t**14 + 3194655744*t**13 - 12830554008*t**12 -
1764930648*t**11 + 231648300*t**10 + 132449592*t**9 + 17211594*t**8
- 2774880*t**7 - 819606*t**6 + 1206*t**5 + 10692*t**4 + 186*t**3
- 48*t**2)*Dt**5 + (4335206400000*t**25 - 17401036800000*t**24 -
54123966627840*t**23 + 30743155556352*t**22 + 170194713600000*t**21 +
139392526282752*t**20 - 44103439460352*t**19 - 153696259003392*t**18 -
101215208083968*t**17 - 8442552180480*t**16 + 23415336529536*t**15
+ 13411706007552*t**14 + 2901058395840*t**13 + 75749861856*t**12 -
76847829600*t**11 - 10342103016*t**10 + 2360603520*t**9 + 1266702984*t**8
+ 158557236*t**7 - 22712346*t**6 - 5949204*t**5 + 15066*t**4 + 57990*t**3
+ 504*t**2 - 210*t)*Dt**4 + (52022476800000*t**23 - 235174625280000*t**22
- 653727018516480*t**21 + 607834202701824*t**20 + 2154097705844736*t**19 +
1219919179825152*t**18 - 737475942629376*t**17 - 1198231459540992*t**16 -
451611388588032*t**15 + 131789136743424*t**14 + 186108289191936*t**13
+ 71489914488576*t**12 + 11769734162304*t**11 + 169707052800*t**10 -
233036093376*t**9 - 373372416*t**8 + 30123663552*t**7 + 9808708032*t**6
+ 839960352*t**5 - 97909632*t**4 - 14716656*t**3 + 391536*t**2 +
70416*t - 1152)*Dt**2 + (41617981440000*t**22 - 198684573696000*t**21 -
524672725155840*t**20 + 585686784540672*t**19 + 1772173319012352*t**18 +
799378452578304*t**17 - 676653069385728*t**16 - 760539449548800*t**15 -
156017339301888*t**14 + 144246562854912*t**13 + 112918479372288*t**12
+ 33988717255680*t**11 + 4071189438720*t**10 - 274517437824*t**9 -
139879082304*t**8 + 16983760320*t**7 + 23389472160*t**6 + 5822194464*t**5
+ 355302000*t**4 - 39458736*t**3 - 2697840*t**2 + 298656*t + 14976)*Dt )

dop[11, 0, 1] = ( 11943936*t**10 + 6469632*t**9 + 9704448*t**8 +
11632896*t**7 + 870912*t**6 + 991872*t**5 - 302400*t**4 - 30096*t**3
- 47952*t**2 - 23760*t - 1800 + (115458048*t**12 + 32265216*t**11 +
78008832*t**10 + 70792704*t**9 - 12960000*t**8 + 10934784*t**7 -
5170464*t**6 - 62592*t**5 - 289296*t**4 - 61428*t**3 + 28584*t**2 +
4320*t - 450)*Dt**2 + (87588864*t**11 + 33343488*t**10 + 63700992*t**9 +
66334464*t**8 - 5640192*t**7 + 8131968*t**6 - 3306240*t**5 - 70704*t**4 -
339408*t**3 - 70800*t**2 + 9000*t + 3600)*Dt + (331776*t**15 + 27648*t**14
+ 193536*t**13 + 110592*t**12 - 41472*t**11 + 27648*t**10 - 18048*t**9 -
352*t**8 + 512*t**7 - 204*t**6 + 156*t**5 - 2*t**4 - 5*t**3)*Dt**5 +
(7133184*t**14 + 995328*t**13 + 4340736*t**12 + 2930688*t**11 -
926208*t**10 + 633600*t**9 - 373056*t**8 - 6624*t**7 + 2744*t**6 -
3984*t**5 + 2976*t**4 + 58*t**3 - 85*t**2)*Dt**4 + (47775744*t**13
+ 9704448*t**12 + 30495744*t**11 + 23991552*t**10 - 6110208*t**9 +
4436352*t**8 - 2354112*t**7 - 33552*t**6 - 47120*t**5 - 24492*t**4 +
16512*t**3 + 1112*t**2 - 390*t)*Dt**3 )

dop[12, 0, 1] = ( 4671406080*t**15 + 18673827840*t**14
+ 34627829760*t**13 + 39329740800*t**12 + 27683819520*t**11 +
8262147840*t**10 - 5100771840*t**9 - 7546716480*t**8 - 4476241920*t**7
- 1632949440*t**6 - 403985760*t**5 - 70684800*t**4 - 8772960*t**3 -
725280*t**2 - 33360*t - 480 + (35035545600*t**17 + 146453299200*t**16
+ 282553958400*t**15 + 331737768960*t**14 + 245359979520*t**13
+ 89146320000*t**12 - 25972016640*t**11 - 55090186080*t**10 -
35257345920*t**9 - 13135609440*t**8 - 3068702640*t**7 - 420888960*t**6
- 18175440*t**5 + 5005680*t**4 + 1134840*t**3 + 96960*t**2 + 1200*t -
240)*Dt**2 + (28028436480*t**16 + 114602803200*t**15 + 216859607040*t**14
+ 250559662080*t**13 + 181093754880*t**12 + 60406329600*t**11 -
25744220160*t**10 - 44769120960*t**9 - 27618846720*t**8 - 10210946880*t**7
- 2471026080*t**6 - 393419520*t**5 - 36988320*t**4 - 612960*t**3 +
359760*t**2 + 47520*t + 1920)*Dt + (6488064*t**21 + 29491200*t**20 +
61038592*t**19 + 75718656*t**18 + 59907072*t**17 + 26649600*t**16
- 151680*t**15 - 9292800*t**14 - 6771328*t**13 - 2487680*t**12 -
425976*t**11 + 26792*t**10 + 28996*t**9 + 5386*t**8 + 184*t**7 - 78*t**6
- 14*t**5 - t**4)*Dt**6 + (15571353600*t**18 + 66512486400*t**17
+ 130732032000*t**16 + 155813913600*t**15 + 117594772480*t**14
+ 45708188800*t**13 - 8747284480*t**12 - 24014061920*t**11 -
15911461760*t**10 - 5944766560*t**9 - 1322103280*t**8 - 142062400*t**7
+ 7670800*t**6 + 5056160*t**5 + 741560*t**4 + 39280*t**3 - 2400*t**2 -
360*t)*Dt**3 + (233570304*t**20 + 1040351232*t**19 + 2117828608*t**18 +
2594394112*t**17 + 2024507392*t**16 + 868413952*t**15 - 47030272*t**14 -
343547968*t**13 - 242847264*t**12 - 90078864*t**11 - 17236248*t**10 -
231312*t**9 + 705064*t**8 + 159436*t**7 + 11564*t**6 - 1000*t**5 -
280*t**4 - 22*t**3)*Dt**5 + (2919628800*t**19 + 12737740800*t**18
+ 25487820800*t**17 + 30810321920*t**16 + 23669433600*t**15 +
9713443200*t**14 - 1113231040*t**13 - 4402556000*t**12 - 3015124960*t**11
- 1124738000*t**10 - 234329240*t**9 - 15849120*t**8 + 4912120*t**7 +
1487180*t**6 + 159500*t**5 + 1080*t**4 - 1600*t**3 - 150*t**2)*Dt**4 )

dop[13, 0, 1] = ( 14400 + 2880*t + 4046400*t**3 - 1486080*t**2
+ 741303898560*t**11 + 1464669786240*t**14 + 930321452160*t**13
+ 45925936865280*t**16 + 5308896960*t**8 - 280556723520*t**12 +
15476451840*t**10 - 13685659200*t**9 - 6609530880*t**7 + 311322240*t**5
- 1555079040*t**6 + 104385600*t**4 - 91961469296640*t**19 +
28579450060800*t**20 - 227224752906240*t**18 + 279069785948160*t**17 -
67780605081600*t**15 + (214345875456000*t**22 - 720831168921600*t**21 -
1291576062320640*t**20 + 2141466580684800*t**19 - 531594796216320*t**18 -
144002049609600*t**17 + 150304577391360*t**16 - 110083765625280*t**15
+ 7494931486080*t**14 + 19251220190400*t**13 - 3597838700040*t**12
- 506787148080*t**11 + 248265557280*t**10 - 139009104720*t**9 -
7681969440*t**8 + 15061902480*t**7 + 819573840*t**6 - 595653840*t**5 -
58639320*t**4 + 9067680*t**3 + 1000080*t**2 - 73440*t - 5760)*Dt**2 +
(95264833536000*t**23 - 327284997120000*t**22 - 487389069803520*t**21 +
942003473264640*t**20 - 379666080645120*t**19 + 55165695096960*t**18 +
59716648448640*t**17 - 95901767792640*t**16 + 18091027143360*t**15 +
16107459300960*t**14 - 4351942923120*t**13 - 704566535040*t**12
+ 265307266680*t**11 - 84346988400*t**10 + 650191560*t**9 +
12994171440*t**8 - 192241320*t**7 - 698135040*t**6 - 24191880*t**5
+ 16263360*t**4 + 945360*t**3 - 183600*t**2 - 7920*t + 720)*Dt**3 +
(171476700364800*t**21 - 564216875458560*t**20 - 1195276913786880*t**19 +
1705999647006720*t**18 - 107182015119360*t**17 - 280437386060160*t**16
+ 88256573568000*t**15 - 34032206168640*t**14 - 3369593053440*t**13
+ 8441537472000*t**12 - 828854706960*t**11 - 143684182080*t**10 +
113318827920*t**9 - 65811248640*t**8 - 11530423440*t**7 + 4907334240*t**6
+ 840122640*t**5 - 100936800*t**4 - 25979040*t**3 + 813600*t**2 +
191520*t - 7200)*Dt + (39693680640*t**26 - 145013234688*t**25 -
103183331328*t**24 + 345168580608*t**23 - 264806461440*t**22 +
264824104752*t**21 - 40068546048*t**20 - 182424972984*t**19
+ 64395889320*t**18 + 39215881192*t**17 - 15773363085*t**16 -
3671223634*t**15 + 1626950899*t**14 + 65720592*t**13 - 65533996*t**12 +
18187900*t**11 - 704922*t**10 - 1822132*t**9 + 126259*t**8 + 76290*t**7 -
2689*t**6 - 1524*t**5 + 6*t**4 + 12*t**3)*Dt**6 + (1428972503040*t**25
- 5116742618112*t**24 - 4862882469888*t**23 + 13207642251264*t**22 -
8651569969152*t**21 + 5984661183408*t**20 - 347550696864*t**19 -
4196545080552*t**18 + 1352583043920*t**17 + 820584506520*t**16
- 308186795910*t**15 - 65526021336*t**14 + 27169263012*t**13 -
220029012*t**12 - 734257344*t**11 + 478205904*t**10 - 27866712*t**9 -
39575508*t**8 + 1660218*t**7 + 1446636*t**6 - 14028*t**5 - 25200*t**4 -
132*t**3 + 180*t**2)*Dt**5 + (17862156288000*t**24 - 62662609843200*t**23
- 75770348037120*t**22 + 172183884641280*t**21 - 92310107811840*t**20
+ 38909478142080*t**19 + 5484412884240*t**18 - 31768543010160*t**17
+ 8600507854920*t**16 + 5669885638440*t**15 - 1891079391060*t**14
- 356819374200*t**13 + 137176222110*t**12 - 15041156100*t**11 -
1318194180*t**10 + 4030277340*t**9 - 203888340*t**8 - 272483820*t**7 +
2331420*t**6 + 8288460*t**5 + 170910*t**4 - 120360*t**3 - 2340*t**2 +
720*t)*Dt**4 )

dop[14, 0, 1] = ( 48000 + 248640*t - 7585920*t**3 +
13920914160*t**5 + 1651507200000*t**20 + 1844014080000*t**19 +
97975221120*t**6 - 5104800*t**2 + 13760559741600*t**11 + 1103767920*t**4
+ 36241208801280*t**16 - 52593272167680*t**13 + 2147413556880*t**8 -
11812773273600*t**18 - 9108668805120*t**17 + 31901752442880*t**15 -
38553131773440*t**14 - 12692057454720*t**12 + 13961526485760*t**10
+ 6681165533280*t**9 + 517516254000*t**7 + (12386304000000*t**22 +
10289817600000*t**21 - 84426927052800*t**20 - 42030962472960*t**19 +
239210635453440*t**18 + 146142841359360*t**17 - 267659455587840*t**16 -
280989954670080*t**15 - 24047044754880*t**14 + 128873702816640*t**13 +
103764252127800*t**12 + 35574257341560*t**11 + 3351325658760*t**10
- 1772242629600*t**9 - 802138904760*t**8 - 193567930920*t**7 -
36220867080*t**6 - 3745392720*t**5 + 115185600*t**4 + 50678640*t**3
+ 788400*t**2 - 276480*t - 7680)*Dt**2 + (2293760000*t**26
+ 594304000*t**25 - 14672224000*t**24 - 3079586560*t**23 +
36889855104*t**22 + 20694439808*t**21 - 45389380480*t**20 -
59141259456*t**19 + 11256190568*t**18 + 65073773180*t**17 +
28450435103*t**16 - 16344055165*t**15 - 18944263365*t**14 -
6822579016*t**13 - 939151521*t**12 + 44881069*t**11 + 29207937*t**10
+ 4821676*t**9 + 818840*t**8 + 36252*t**7 - 21382*t**6 - 2684*t**5 +
76*t**4 + 16*t**3)*Dt**6 + (9909043200000*t**21 + 9647969280000*t**20 -
69104396390400*t**19 - 43180640378880*t**18 + 203751249592320*t**17 +
148512476682240*t**16 - 222963930378240*t**15 - 260174765905920*t**14
- 43887863155200*t**13 + 87579956156640*t**12 + 80381771462160*t**11
+ 35700946874880*t**10 + 9661725582960*t**9 + 1621279542960*t**8 +
147067804560*t**7 - 10951813920*t**6 - 9154669680*t**5 - 1751681280*t**4 -
84452640*t**3 + 7109760*t**2 + 370560*t - 38400)*Dt + (5505024000000*t**23
+ 3786521600000*t**22 - 36771152998400*t**21 - 14400987156480*t**20 +
100274961459200*t**19 + 53196702730240*t**18 - 114385977553920*t**17 -
115262922679040*t**16 + 205935597440*t**15 + 71841188935200*t**14 +
49806073285040*t**13 + 8920106064640*t**12 - 5058052229520*t**11
- 3392287624160*t**10 - 920714096920*t**9 - 158317050000*t**8 -
21003000880*t**7 - 878332160*t**6 + 408409320*t**5 + 62335160*t**4 -
672800*t**3 - 471840*t**2 - 6240*t + 960)*Dt**3 + (82575360000*t**25
+ 33195904000*t**24 - 534244236800*t**23 - 131866030080*t**22 +
1368479432192*t**21 + 686160537856*t**20 - 1630983457536*t**19 -
1857769500480*t**18 + 272811228128*t**17 + 1807968210228*t**16
+ 917060551590*t**15 - 266113782704*t**14 - 421981939470*t**13 -
169129128306*t**12 - 30156705196*t**11 - 1662841128*t**10 + 242048430*t**9
+ 95250760*t**8 + 24465870*t**7 + 2031950*t**6 - 308840*t**5 - 47544*t**4
+ 856*t**3 + 240*t**2)*Dt**5 + (1032192000000*t**24 + 562460800000*t**23
- 6775416422400*t**22 - 2084274618880*t**21 + 17849807284480*t**20 +
8789355240960*t**19 - 20768558904640*t**18 - 21563739738400*t**17 +
1781949481680*t**16 + 17384258038780*t**15 + 10303797222820*t**14
- 365584631640*t**13 - 2772275657590*t**12 - 1295363154250*t**11 -
283897071540*t**10 - 34145602970*t**9 - 2325775490*t**8 + 351465900*t**7 +
186961520*t**6 + 20718640*t**5 - 1123200*t**4 - 253720*t**3 + 1080*t**2 +
960*t)*Dt**4 )

dop[15, 0, 1] = ( 61931520*t**11 + 33619968*t**10 + 38486016*t**9 +
22717440*t**8 - 13040640*t**7 - 2658816*t**6 + 112896*t**5 - 211392*t**4
- 91584*t**3 - 20448*t**2 - 5040*t + (309657600*t**13 + 218824704*t**12
+ 151953408*t**11 + 104228352*t**10 - 62996736*t**9 - 35173248*t**8 +
9019008*t**7 - 752160*t**6 - 521808*t**5 + 169128*t**4 + 10584*t**3 +
3528*t**2 + 2790*t - 450)*Dt**2 + (309657600*t**12 + 193462272*t**11 +
173850624*t**10 + 109440000*t**9 - 63663360*t**8 - 26060544*t**7 +
5226624*t**6 - 725568*t**5 - 580176*t**4 - 9072*t**3 + 33480*t**2
+ 2700*t + 900)*Dt + (516096*t**16 + 491520*t**15 + 110592*t**14 +
139776*t**13 - 106880*t**12 - 83200*t**11 + 27456*t**10 - 2376*t**9 +
392*t**8 + 1500*t**7 - 456*t**6 + 3*t**5 + 18*t**4 - 5*t**3)*Dt**5 +
(12902400*t**15 + 11231232*t**14 + 4091904*t**13 + 3803136*t**12 -
2647936*t**11 - 1937280*t**10 + 608064*t**9 - 50400*t**8 - 3648*t**7
+ 27048*t**6 - 6636*t**5 + 108*t**4 + 333*t**3 - 85*t**2)*Dt**4 +
(103219200*t**14 + 81395712*t**13 + 42246144*t**12 + 32706048*t**11 -
21015936*t**10 - 13909248*t**9 + 4037184*t**8 - 319104*t**7 - 114936*t**6
+ 132672*t**5 - 20316*t**4 + 1098*t**3 + 1752*t**2 - 390*t)*Dt**3 )

dop[16, 0, 1] = ( 9157017600*t**16 - 69268193280*t**15
- 181007032320*t**14 - 169368698880*t**13 - 33680977920*t**12 +
56649185280*t**11 + 42901032960*t**10 + 12449410560*t**9 - 1342978560*t**8
- 3617645760*t**7 - 1609325280*t**6 - 284309280*t**5 - 32996160*t**4 -
6288480*t**3 - 560880*t**2 - 21600*t + 2160 + (68677632000*t**18 -
458580787200*t**17 - 1478937968640*t**16 - 1600798924800*t**15 -
548562032640*t**14 + 363817175040*t**13 + 383075735040*t**12
+ 118384035840*t**11 - 7621493760*t**10 - 27594455040*t**9 -
12940927200*t**8 - 2216809440*t**7 - 61915680*t**6 + 17266320*t**5 +
2941560*t**4 + 637920*t**3 + 147240*t**2 + 2880*t - 720)*Dt**2 +
(54942105600*t**17 - 391236894720*t**16 - 1136972021760*t**15 -
1146593710080*t**14 - 317896888320*t**13 + 317665198080*t**12
+ 281554744320*t**11 + 83985246720*t**10 - 6946467840*t**9 -
21752890560*t**8 - 10025619840*t**7 - 1755614160*t**6 - 133510320*t**5
- 15673680*t**4 - 817200*t**3 + 281520*t**2 + 80640*t + 720)*Dt +
(12718080*t**22 - 62355456*t**21 - 305636352*t**20 - 428084736*t**19 -
226704384*t**18 + 34497536*t**17 + 97547584*t**16 + 36955584*t**15 -
874880*t**14 - 6361896*t**13 - 2808864*t**12 - 337974*t**11 + 112131*t**10
+ 44892*t**9 + 3779*t**8 - 962*t**7 - 99*t**6 + 4*t**5 - 3*t**4)*Dt**6
+ (30523392000*t**19 - 190273536000*t**18 - 680320696320*t**17
- 787856547840*t**16 - 314004633600*t**15 + 144706990080*t**14
+ 184728422400*t**13 + 59582476800*t**12 - 3018309120*t**11 -
12634401600*t**10 - 5934579840*t**9 - 975600960*t**8 + 30056760*t**7 +
27846240*t**6 + 3277440*t**5 + 204960*t**4 + 58320*t**3 + 2640*t**2 -
1080*t)*Dt**3 + (457850880*t**21 - 2447898624*t**20 - 10776471552*t**19 -
14189099520*t**18 - 6960006144*t**17 + 1575404544*t**16 + 3246635136*t**15
+ 1161107136*t**14 - 36259968*t**13 - 211481736*t**12 - 96067440*t**11 -
13355466*t**10 + 2674380*t**9 + 1173246*t**8 + 109434*t**7 - 14802*t**6
- 1236*t**5 + 102*t**4 - 66*t**3)*Dt**5 + (5723136000*t**20 -
33137510400*t**19 - 131380485120*t**18 - 162384890880*t**17
- 72628884480*t**16 + 23576413440*t**15 + 37509780480*t**14 +
12707810880*t**13 - 507984960*t**12 - 2481249480*t**11 - 1151984580*t**10
- 177108600*t**9 + 18493710*t**8 + 9651120*t**7 + 997350*t**6 - 34440*t**5
+ 2370*t**4 + 840*t**3 - 450*t**2)*Dt**4 )

dop[17, 0, 1] = ( 524880*t**5 + 3324240*t**4 - 388800*t**3 -
90720*t**2 + 7200 + (3936600*t**7 + 24494400*t**6 + 1409400*t**5 -
3645000*t**4 + 152280*t**3 + 24480*t**2 + 25200*t - 3600)*Dt**2 +
(3149280*t**6 + 19770480*t**5 - 544320*t**4 - 1704240*t**3 + 51840*t**2 +
43200*t)*Dt + (1749600*t**8 + 10789200*t**7 + 1490400*t**6 - 2315520*t**5
+ 113400*t**4 - 13560*t**3 + 33360*t**2 - 4800*t)*Dt**3 + (729*t**11
+ 4374*t**10 + 1539*t**9 - 1863*t**8 + 108*t**7 - 57*t**6 + 68*t**5 -
10*t**4)*Dt**6 + (328050*t**9 + 2004750*t**8 + 429300*t**7 - 567540*t**6 +
30510*t**5 - 8910*t**4 + 12420*t**3 - 1800*t**2)*Dt**4 + (26244*t**10 +
158922*t**9 + 45360*t**8 - 56214*t**7 + 3186*t**6 - 1338*t**5 + 1644*t**4
- 240*t**3)*Dt**5 )

dop[18, 0, 1] = ( 288*t + 24 + (1152*t**2 + 168*t - 24)*Dt +
(864*t**3 + 180*t**2 - 36*t)*Dt**2 + (192*t**4 + 52*t**3 - 12*t**2)*Dt**3
+ (12*t**5 + 4*t**4 - t**3)*Dt**4 )

dop[19, 0, 1] = ( 96*t + (144*t**3 - 9*t)*Dt**2 + (16*t**4 -
t**2)*Dt**3 + (288*t**2 - 15)*Dt )

dop[1, 1, 1] = ( 64*t + 12 + (128*t**3 + 8*t**2 - 6*t)*Dt**2 +
(16*t**4 - t**2)*Dt**3 + (224*t**2 + 28*t - 6)*Dt )

dop[2, 1, 1] = ( 128*t**2 - 48*t - 4 + (448*t**4 - 112*t**3 -
20*t**2 + 5*t)*Dt**2 + (64*t**5 - 16*t**4 - 4*t**3 + t**2)*Dt**3 +
(640*t**3 - 176*t**2 - 20*t + 4)*Dt )

dop[3, 1, 1] = ( 3456*t**5 - 1152*t**4 - 1104*t**3 + 96*t**2 +
132*t + 8 + (12096*t**7 - 576*t**6 - 4080*t**5 + 348*t**3 + 4*t**2 -
5*t)*Dt**2 + (1728*t**8 - 624*t**6 + 52*t**4 - t**2)*Dt**3 + (17280*t**6 -
2304*t**5 - 5520*t**4 + 96*t**3 + 516*t**2 + 16*t - 4)*Dt )

dop[4, 1, 1] = ( 384*t**3 - 72*t**2 - 144*t - 12 + (576*t**5 +
200*t**4 - 252*t**3 - 33*t**2 + 5*t)*Dt**2 + (64*t**6 + 40*t**5 - 30*t**4
- 5*t**3 + t**2)*Dt**3 + (1152*t**4 + 88*t**3 - 468*t**2 - 48*t + 4)*Dt )

dop[5, 1, 1] = ( 20643840*t**11 - 8650752*t**10 + 24975360*t**9 +
1711488*t**8 + 52224*t**7 - 978144*t**6 - 179040*t**5 - 52176*t**4 -
12552*t**3 - 4944*t**2 - 552*t - 96 + (103219200*t**12 - 53280768*t**11
+ 115279872*t**10 - 2172288*t**9 - 10069632*t**8 - 2219040*t**7 -
435456*t**6 + 269016*t**5 - 25992*t**4 + 17496*t**3 + 5496*t**2 +
648*t + 96)*Dt + (103219200*t**13 - 63307776*t**12 + 99222528*t**11 -
4771776*t**10 - 27972480*t**9 + 4192752*t**8 - 1230384*t**7 + 880272*t**6
- 126672*t**5 + 52656*t**4 - 1620*t**3 - 600*t**2 - 96*t)*Dt**2 +
(172032*t**16 - 155648*t**15 + 20480*t**14 + 78016*t**13 - 188224*t**12 +
78144*t**11 - 6528*t**10 + 701*t**9 + 2131*t**8 - 1183*t**7 + 97*t**6 -
30*t**5 + 12*t**4)*Dt**5 + (4300800*t**15 - 3473408*t**14 + 1988608*t**13
+ 777472*t**12 - 3375296*t**11 + 1221168*t**10 - 155184*t**9 + 62906*t**8
+ 8591*t**7 - 5998*t**6 + 113*t**5 - 396*t**4 + 84*t**3)*Dt**4 +
(34406400*t**14 - 24444928*t**13 + 25568256*t**12 + 606272*t**11 -
17373888*t**10 + 4926192*t**9 - 857568*t**8 + 477496*t**7 - 48240*t**6 +
13480*t**5 - 3036*t**4 - 1308*t**3 + 48*t**2)*Dt**3 )

dop[6, 1, 1] = ( 1957478400*t**15 + 5173051392*t**14
+ 8641327104*t**13 + 11840664576*t**12 + 12601384704*t**11
+ 10526148096*t**10 + 7011623808*t**9 + 3718155264*t**8 +
1655854800*t**7 + 595362768*t**6 + 156563304*t**5 + 30072960*t**4
+ 4664736*t**3 + 522000*t**2 + 41256*t + 864 + (9787392000*t**17
+ 26359382016*t**16 + 39654881280*t**15 + 48619728384*t**14 +
45503857536*t**13 + 30677616384*t**12 + 14272094208*t**11 +
3439764864*t**10 - 346860984*t**9 - 838397256*t**8 - 547681056*t**7
- 184426704*t**6 - 36306912*t**5 - 4593816*t**4 - 257472*t**3 +
30816*t**2 + 10296*t + 720)*Dt**2 + (16312320*t**20 + 45167616*t**19 +
45732864*t**18 + 24396544*t**17 - 1517504*t**16 - 30982592*t**15 -
42343824*t**14 - 32941040*t**13 - 16946844*t**12 - 5575272*t**11 -
1319911*t**10 - 96520*t**9 + 77896*t**8 + 31496*t**7 + 5138*t**6 -
216*t**5 - 132*t**4 - 16*t**3 - 3*t**2)*Dt**5 + (9787392000*t**16
+ 26112319488*t**15 + 41889927168*t**14 + 54980441088*t**13
+ 55713268992*t**12 + 43241357568*t**11 + 25864050048*t**10 +
11587592064*t**9 + 4073300784*t**8 + 981797472*t**7 + 38884824*t**6 -
53913936*t**5 - 18786816*t**4 - 4166784*t**3 - 605736*t**2 - 62928*t -
2016)*Dt + (3262464000*t**18 + 8868814848*t**17 + 12167165952*t**16 +
13304638976*t**15 + 10842065280*t**14 + 5208128128*t**13 + 403021568*t**12
- 1672016768*t**11 - 1488001720*t**10 - 751559584*t**9 - 300211528*t**8 -
75489528*t**7 - 9313472*t**6 + 258000*t**5 + 320744*t**4 + 62312*t**3 +
7112*t**2 + 288*t - 48)*Dt**3 + (407808000*t**19 + 1118896128*t**18 +
1351240704*t**17 + 1193989120*t**16 + 707269184*t**15 - 46215232*t**14 -
514787152*t**13 - 541463264*t**12 - 327102652*t**11 - 129798372*t**10 -
40915619*t**9 - 7485544*t**8 + 109540*t**7 + 409432*t**6 + 100374*t**5 +
9272*t**4 + 176*t**3 - 68*t**2 - 27*t)*Dt**4 )

dop[7, 1, 1] = ( 35831808*t**10 - 31186944*t**9 + 42163200*t**8 +
11639808*t**7 + 4981248*t**6 - 981504*t**5 - 809280*t**4 + 72576*t**3
- 177408*t**2 + 8064*t - 3168 + (346374144*t**12 - 454643712*t**11 +
545398272*t**10 - 166067712*t**9 + 59053824*t**8 - 32668800*t**7 +
2167392*t**6 + 54912*t**5 - 687744*t**4 + 500616*t**3 - 31176*t**2 -
5004*t + 288)*Dt**2 + (143327232*t**13 - 222621696*t**12 + 257753088*t**11
- 122575104*t**10 + 36213888*t**9 - 19897728*t**8 + 1942656*t**7 +
70768*t**6 - 100456*t**5 + 254712*t**4 - 35124*t**3 - 7404*t**2 +
1116*t - 48)*Dt**3 + (995328*t**15 - 2036736*t**14 + 2253312*t**13 -
1520640*t**12 + 491520*t**11 - 212992*t**10 + 9536*t**9 + 21184*t**8 -
1888*t**7 + 2420*t**6 - 990*t**5 - 108*t**4 + 57*t**3 - 3*t**2)*Dt**5 +
(262766592*t**11 - 284000256*t**10 + 358041600*t**9 - 21846528*t**8 +
33115392*t**7 - 13748736*t**6 - 1184640*t**5 + 651744*t**4 - 894672*t**3
+ 278496*t**2 - 7272*t + 2880)*Dt + (21399552*t**14 - 38486016*t**13
+ 43416576*t**12 - 25803264*t**11 + 7762176*t**10 - 3848960*t**9 +
337088*t**8 + 143168*t**7 - 7128*t**6 + 45328*t**5 - 11304*t**4 -
1854*t**3 + 540*t**2 - 27*t)*Dt**4 )

dop[8, 1, 1] = ( (3019898880*t**15 + 16087252992*t**14
+ 34639183872*t**13 + 45517897728*t**12 + 45817583616*t**11
+ 39451150848*t**10 + 28301194752*t**9 + 15652920000*t**8 +
6314028384*t**7 + 1793194800*t**6 + 343002600*t**5 + 39229632*t**4 +
1245888*t**3 - 365184*t**2 - 59904*t - 3168)*Dt**2 + (1509949440*t**14
+ 7977566208*t**13 + 16813916160*t**12 + 21954035712*t**11 +
22623148032*t**10 + 20245324800*t**9 + 14949742848*t**8 + 8421174912*t**7
+ 3475673664*t**6 + 1038989856*t**5 + 224642304*t**4 + 35000064*t**3 +
3852288*t**2 + 279936*t + 10368)*Dt + (12582912*t**18 + 68681728*t**17 +
155811840*t**16 + 207601664*t**15 + 195727360*t**14 + 149026816*t**13
+ 95639040*t**12 + 48602368*t**11 + 17355072*t**10 + 3488032*t**9 -
12384*t**8 - 211104*t**7 - 51308*t**6 - 3166*t**5 + 694*t**4 + 142*t**3 +
9*t**2)*Dt**5 + (1509949440*t**16 + 8109686784*t**15 + 17802067968*t**14
+ 23520870400*t**13 + 23149011968*t**12 + 19158059008*t**11 +
13301338240*t**10 + 7190601344*t**9 + 2813245888*t**8 + 742452448*t**7
+ 114502520*t**6 + 3743104*t**5 - 2371280*t**4 - 511432*t**3 -
41456*t**2 - 144*t + 144)*Dt**3 + (251658240*t**17 + 1362624512*t**16 +
3043557376*t**15 + 4039819264*t**14 + 3890388992*t**13 + 3090608128*t**12
+ 2068002816*t**11 + 1087134464*t**10 + 408737536*t**9 + 96911616*t**8 +
9340256*t**7 - 1930544*t**6 - 792672*t**5 - 101592*t**4 - 1470*t**3 +
938*t**2 + 81*t)*Dt**4 )

dop[9, 1, 1] = ( 5472 + 54720*t + 752335446528*t**14
- 73165152*t**5 + 741760174080*t**15 + 41016282624*t**13 -
622571332608*t**16 - 42345186432*t**11 - 159365026368*t**12 +
573120*t**2 + 6889824*t**3 + 5746195296*t**8 + 979067136*t**10 +
13050737184*t**9 - 919703136*t**7 - 815679552*t**6 + 27207936*t**4 -
2819533307904*t**18 + 1017623347200*t**19 - 6409792880640*t**17
+ (8480194560*t**24 - 10534551552*t**23 - 50391908352*t**22
- 50341446144*t**21 - 45028601856*t**20 - 23078165376*t**19 +
15152198016*t**18 + 14141918376*t**17 + 406236968*t**16 - 2356214710*t**15
- 725447438*t**14 + 214114051*t**13 + 155870421*t**12 - 15954357*t**11 -
19631947*t**10 + 572822*t**9 + 1454362*t**8 + 41248*t**7 - 54948*t**6 -
3777*t**5 + 873*t**4 + 75*t**3 - 3*t**2)*Dt**5 + (5088116736000*t**21
- 10986892296192*t**20 - 32040387330048*t**19 - 17762607797760*t**18
- 5791079218176*t**17 + 1729627137792*t**16 + 3050844311808*t**15
+ 692771704416*t**14 - 200248782816*t**13 - 224736807168*t**12
- 29508738504*t**11 + 56107923720*t**10 + 14537747940*t**9 -
6517524624*t**8 - 2424464784*t**7 + 137116176*t**6 + 139005576*t**5 +
13825152*t**4 - 1104264*t**3 - 199224*t**2 - 4716*t - 432)*Dt**2 +
(5088116736000*t**20 - 12542279417856*t**19 - 32164167647232*t**18 -
11094615677952*t**17 - 611045351424*t**16 + 3533217080832*t**15 +
1712902740480*t**14 - 212101200000*t**13 - 292228465152*t**12
- 125692983312*t**11 + 35148192384*t**10 + 48671287656*t**9 +
3330260544*t**8 - 5673118656*t**7 - 1226485344*t**6 + 134510448*t**5
+ 69515424*t**4 + 7222032*t**3 + 223200*t**2 + 45288*t + 4896)*Dt +
(1696038912000*t**22 - 3143835058176*t**21 - 10559207817216*t**20
- 7716307170816*t**19 - 3960532002816*t**18 - 569564411136*t**17
+ 1502965006080*t**16 + 738999673632*t**15 - 2148685088*t**14
- 129094561808*t**13 - 44496399080*t**12 + 20144496184*t**11 +
10590149700*t**10 - 2273963052*t**9 - 1429750336*t**8 + 49921088*t**7 +
89687336*t**6 + 6973640*t**5 - 1790280*t**4 - 234168*t**3 + 7188*t**2
+ 852*t - 48)*Dt**3 + (212004864000*t**23 - 328171585536*t**22 -
1294828167168*t**21 - 1136901888000*t**20 - 789052972032*t**19 -
286047456384*t**18 + 266927151360*t**17 + 195531194352*t**16
+ 7769608856*t**15 - 30604648840*t**14 - 11371548118*t**13 +
3219579506*t**12 + 2392492355*t**11 - 307528792*t**10 - 301613595*t**9 +
8388972*t**8 + 20719246*t**7 + 1074592*t**6 - 623268*t**5 - 59166*t**4 +
6759*t**3 + 624*t**2 - 27*t)*Dt**4 )

dop[10, 1, 1] = ( - 22464 - 451296*t - 64845759168*t**8 -
8567520*t**2 - 140485008*t**3 - 495281498112*t**10 - 109934770176*t**9
- 53271954240*t**7 - 26356354176*t**6 - 6994180227072*t**16 +
1235817283584*t**15 + 5584717234176*t**14 + 1907260735488*t**13
- 1376741382144*t**12 - 7768879584*t**5 - 1399425761280*t**11 +
227082240000*t**19 + 1090466611200*t**18 - 4130053816320*t**17
- 1370419584*t**4 + (378470400000*t**22 + 1821691084800*t**21 -
4964213882880*t**20 - 12009478135808*t**19 - 4381901037568*t**18 +
8936076752896*t**17 + 11847464069120*t**16 + 4115996062720*t**15 -
2219866571776*t**14 - 2632683934976*t**13 - 1121810512000*t**12
- 331856938400*t**11 - 129562335520*t**10 - 61796701736*t**9 -
21296426392*t**8 - 4208335952*t**7 - 300047680*t**6 + 45411880*t**5
+ 10529090*t**4 + 535888*t**3 - 18106*t**2 - 1020*t + 48)*Dt**3 +
(1135411200000*t**21 + 5460826521600*t**20 - 16785832181760*t**19 -
36057594863616*t**18 - 7287121182720*t**17 + 27418579642368*t**16 +
26356460402688*t**15 + 4411389139968*t**14 - 7504283639808*t**13 -
5923884943104*t**12 - 2207397164160*t**11 - 718441676064*t**10
- 351342324480*t**9 - 173136422904*t**8 - 57960776808*t**7 -
11637201048*t**6 - 1161271776*t**5 - 2986416*t**4 + 10037526*t**3
+ 711192*t**2 + 15354*t + 792)*Dt**2 + (1135411200000*t**20 +
5456579788800*t**19 - 18705041326080*t**18 - 35704801148928*t**17 -
845976502272*t**16 + 27816503623680*t**15 + 17768620793856*t**14 -
1927605172224*t**13 - 7625766414336*t**12 - 4160593317888*t**11
- 1282076116992*t**10 - 484808936256*t**9 - 302844173952*t**8 -
153142047744*t**7 - 49288460160*t**6 - 9650682192*t**5 - 1077476208*t**4 -
60418992*t**3 - 1875024*t**2 - 158760*t - 9216)*Dt + (1892352000*t**24
+ 9122611200*t**23 - 18640527360*t**22 - 58034765824*t**21 -
38532706304*t**20 + 41965621248*t**19 + 94071750144*t**18
+ 56013030400*t**17 - 5294450432*t**16 - 21996570880*t**15 -
10952369952*t**14 - 2543280896*t**13 - 537441232*t**12 - 276140992*t**11
- 112877022*t**10 - 19917936*t**9 + 1121240*t**8 + 1056920*t**7 +
134035*t**6 - 5391*t**5 - 2254*t**4 - 103*t**3 + 3*t**2)*Dt**5 +
(47308800000*t**23 + 227888332800*t**22 - 542727905280*t**21 -
1484019662848*t**20 - 767620100096*t**19 + 1085290090496*t**18 +
1896743070208*t**17 + 918495748096*t**16 - 215512785664*t**15
- 427218085376*t**14 - 200103936864*t**13 - 53308965120*t**12 -
16198105488*t**11 - 7684582384*t**10 - 2788409498*t**9 - 526917856*t**8 -
11674372*t**7 + 14725960*t**6 + 2406665*t**5 + 42072*t**4 - 17460*t**3 -
836*t**2 + 27*t)*Dt**4 )

dop[11, 1, 1] = ( 3981312*t**10 - 1990656*t**9 - 414720*t**8 -
967680*t**7 - 1534464*t**6 - 1147392*t**5 - 470592*t**4 - 204672*t**3
- 43200*t**2 - 15552*t - 288 + (38486016*t**12 - 24883200*t**11 +
4879872*t**10 - 9658368*t**9 - 9879552*t**8 - 5091072*t**7 - 2474016*t**6
- 750240*t**5 - 98064*t**4 - 17952*t**3 + 14328*t**2 + 1872*t - 144)*Dt**2
+ (29196288*t**11 - 17252352*t**10 + 1050624*t**9 - 7165440*t**8 -
9167616*t**7 - 5509632*t**6 - 2486400*t**5 - 910560*t**4 - 171792*t**3 -
48768*t**2 + 6696*t + 1152)*Dt + (110592*t**15 - 82944*t**14 + 32256*t**13
- 29184*t**12 - 15360*t**11 - 4096*t**10 - 3520*t**9 - 128*t**8 +
352*t**7 + 84*t**6 + 38*t**5 - 2*t**4 - 3*t**3)*Dt**5 + (2377728*t**14 -
1714176*t**13 + 589824*t**12 - 620544*t**11 - 404736*t**10 - 149504*t**9
- 92992*t**8 - 13504*t**7 + 4264*t**6 + 1464*t**5 + 980*t**4 + 40*t**3
- 42*t**2)*Dt**4 + (15925248*t**13 - 10948608*t**12 + 3096576*t**11 -
4084992*t**10 - 3332736*t**9 - 1486080*t**8 - 790272*t**7 - 183888*t**6 -
1240*t**5 + 3728*t**4 + 6772*t**3 + 632*t**2 - 156*t)*Dt**3 )

dop[12, 1, 1] = ( 336199680*t**13 + 313786368*t**12 -
1210217472*t**11 - 3548321280*t**10 - 4783376256*t**9 - 4124379072*t**8
- 2480226144*t**7 - 1072018992*t**6 - 335901408*t**5 - 75983808*t**4
- 12162480*t**3 - 1310520*t**2 - 84624*t - 2376 + (560332800*t**16 +
825556992*t**15 - 1355324928*t**14 - 5348057088*t**13 - 7780320192*t**12 -
6899846208*t**11 - 4143029280*t**10 - 1720573776*t**9 - 478739268*t**8 -
77625960*t**7 - 2163072*t**6 + 2046468*t**5 + 448914*t**4 + 32856*t**3
- 1266*t**2 - 264*t)*Dt**3 + (1680998400*t**15 + 2174091264*t**14
- 4747562496*t**13 - 16666344192*t**12 - 23636082240*t**11 -
20819892576*t**10 - 12570271728*t**9 - 5338318008*t**8 - 1575498888*t**7
- 303671856*t**6 - 30034608*t**5 + 1205712*t**4 + 793548*t**3 +
93012*t**2 + 2436*t - 216)*Dt**2 + (1680998400*t**14 + 1871511552*t**13
- 5409266688*t**12 - 17233136640*t**11 - 23830748544*t**10 -
20797863168*t**9 - 12565055616*t**8 - 5408073408*t**7 - 1659219672*t**6
- 354195312*t**5 - 48822960*t**4 - 3260160*t**3 + 144324*t**2 +
46176*t + 2700)*Dt + (2801664*t**18 + 5136384*t**17 - 4405248*t**16 -
24409088*t**15 - 37469440*t**14 - 33463168*t**13 - 19594592*t**12 -
7552624*t**11 - 1704112*t**10 - 80992*t**9 + 77842*t**8 + 24313*t**7
+ 2423*t**6 - 224*t**5 - 77*t**4 - 6*t**3)*Dt**5 + (70041600*t**17 +
115802112*t**16 - 140187648*t**15 - 640398336*t**14 - 956394240*t**13 -
852014848*t**12 - 506408864*t**11 - 203715664*t**10 - 52052064*t**9 -
6097608*t**8 + 823806*t**7 + 451471*t**6 + 67754*t**5 + 1366*t**4 -
792*t**3 - 78*t**2)*Dt**4 )

dop[13, 1, 1] = ( - 4032 - 7920*t + 145395857520*t**12 +
1029554784*t**7 + 28305888*t**5 - 10121318928*t**10 - 3909936*t**3 -
41232*t**2 - 12333461616*t**9 + 351033312384*t**13 - 1370282987520*t**17
+ 576480384*t**6 + 352943308800*t**19 + 85950643104*t**14 +
(1764716544000*t**21 + 8020974108672*t**20 - 8833369282560*t**19 -
6067557491904*t**18 + 119980897920*t**17 - 568717488144*t**16 +
1341361576512*t**15 + 662136672672*t**14 - 6116964444*t**13 -
123074175024*t**12 - 110724077904*t**11 + 3452951928*t**10 +
22744857864*t**9 + 1882124784*t**8 - 1849777344*t**7 - 278291952*t**6
+ 61252596*t**5 + 14499120*t**4 - 555408*t**3 - 223272*t**2 +
5760*t + 864)*Dt**2 + (588238848000*t**22 + 2460522479616*t**21 -
3203764254720*t**20 - 1102025822400*t**19 + 65577658752*t**18 -
369891500688*t**17 + 773359948800*t**16 + 352785710808*t**15
- 89996725788*t**14 - 107380002060*t**13 - 43999498848*t**12 +
14226659944*t**11 + 12538329400*t**10 - 461494304*t**9 - 1298805088*t**8
- 70997192*t**7 + 61831444*t**6 + 6881860*t**5 - 1324416*t**4 -
183960*t**3 + 15024*t**2 + 1176*t - 96)*Dt**3 + (2941194240*t**24
+ 10171256832*t**23 - 17901720576*t**22 + 1598920128*t**21 -
5208329088*t**20 - 3077716896*t**19 + 12401046096*t**18 + 3559925085*t**17
- 3316551675*t**16 - 1578147513*t**15 + 35012711*t**14 + 321803908*t**13
+ 100231872*t**12 - 33116504*t**11 - 16553312*t**10 + 1610821*t**9
+ 1195205*t**8 - 17297*t**7 - 43569*t**6 - 1286*t**5 + 790*t**4 +
34*t**3 - 6*t**2)*Dt**5 + (1764716544000*t**20 + 8660380778496*t**19
- 7913410007040*t**18 - 9251725868928*t**17 - 1140380847360*t**16
- 29579999904*t**15 + 1200172323456*t**14 + 519303506040*t**13
+ 51793168752*t**12 - 29834807472*t**11 - 77543942544*t**10 -
15398613504*t**9 + 11230898976*t**8 + 2902863456*t**7 - 540750048*t**6 -
209591592*t**5 + 263856*t**4 + 6119472*t**3 + 359280*t**2 - 13968*t +
2880)*Dt + (73529856000*t**23 + 280923365376*t**22 - 426965852160*t**21
- 40193839872*t**20 - 37104463872*t**19 - 65282112432*t**18
+ 178945878432*t**17 + 66397482426*t**16 - 36626975349*t**15 -
25300952184*t**14 - 4314908037*t**13 + 4395804200*t**12 + 2165392896*t**11
- 340867200*t**10 - 277030384*t**9 + 5663842*t**8 + 16503411*t**7
+ 682416*t**6 - 487365*t**5 - 34228*t**4 + 7346*t**3 + 392*t**2 -
54*t)*Dt**4 - 21624528*t**4 - 3827389536*t**8 - 721369048320*t**15 +
1859957489664*t**18 - 2572243247232*t**16 + 5107950288*t**11 )

dop[14, 1, 1] = ( - 21024 - 349992*t - 1376962416*t**4 -
920661288960*t**17 + 7168883564544*t**16 + 2924371476480*t**15 -
6840240337920*t**14 - 6000754504320*t**13 + 211386847104*t**12
+ 2064462983616*t**10 + 648323624976*t**9 - 73559540112*t**7 -
36241571424*t**6 + 21753502464*t**8 - 9218391864*t**5 - 118411704*t**3
- 6132528*t**2 + 216222720000*t**19 - 1982362214400*t**18 +
2941986918816*t**11 + (360371200000*t**22 - 3120780697600*t**21 +
53143260160*t**20 + 9902231092736*t**19 + 1117140897280*t**18 -
9668721361408*t**17 - 7570746688320*t**16 + 2426080907616*t**15 +
6301324450464*t**14 + 2427079236976*t**13 - 768956495248*t**12
- 891960282296*t**11 - 262695331280*t**10 + 2896085696*t**9 +
25622813248*t**8 + 7871074344*t**7 + 829914492*t**6 - 59670120*t**5
- 19999640*t**4 - 890260*t**3 + 75124*t**2 + 2988*t - 240)*Dt**3 +
(1081113600000*t**21 - 9545498419200*t**20 - 1221654266880*t**19 +
31642699118592*t**18 + 5529614039040*t**17 - 29763428212992*t**16 -
22166099970240*t**15 + 4435749160512*t**14 + 14898468040608*t**13
+ 7756172867136*t**12 + 357781945440*t**11 - 1194409654176*t**10 -
525871809264*t**9 - 58511324064*t**8 + 30265768344*t**7 + 13969231464*t**6
+ 2325320220*t**5 + 108361824*t**4 - 13499664*t**3 - 1366728*t**2 +
13356*t + 2592)*Dt**2 + (1081113600000*t**20 - 9728654745600*t**19 -
2809233008640*t**18 + 33690237213696*t**17 + 9282585292800*t**16 -
31506375252480*t**15 - 24587344736640*t**14 + 2417920495296*t**13 +
13522884960864*t**12 + 8691169786464*t**11 + 2134068254064*t**10
- 264915296064*t**9 - 360401586288*t**8 - 109576175136*t**7 -
8552867016*t**6 + 3719108016*t**5 + 1196625144*t**4 + 145307328*t**3
+ 7992552*t**2 + 285264*t + 17280)*Dt + (1801856000*t**24 -
14993382400*t**23 + 3836856320*t**22 + 43648062208*t**21 +
6154083840*t**20 - 50989450944*t**19 - 54577076576*t**18 +
27002115888*t**17 + 58752540732*t**16 + 11074917368*t**15 -
18744006450*t**14 - 11943827034*t**13 - 2005747424*t**12 + 584373384*t**11
+ 346479237*t**10 + 58247942*t**9 - 3206910*t**8 - 2603142*t**7 -
259585*t**6 + 22663*t**5 + 4831*t**4 + 67*t**3 - 15*t**2)*Dt**5 +
(45046400000*t**23 - 382466073600*t**22 + 55584130560*t**21 +
1161962893056*t**20 + 114180049280*t**19 - 1219597532096*t**18 -
1092838317120*t**17 + 463823944976*t**16 + 1068425564644*t**15
+ 291909440608*t**14 - 253868291970*t**13 - 195119601168*t**12 -
43831671092*t**11 + 5466223528*t**10 + 5643682337*t**9 + 1307241942*t**8 +
52460590*t**7 - 30653398*t**6 - 4825257*t**5 + 17406*t**4 + 42543*t**3 +
866*t**2 - 135*t)*Dt**4 )

dop[15, 1, 1] = ( 35389440*t**11 - 33030144*t**10 - 9289728*t**9
- 23841792*t**8 - 27162624*t**7 - 12344832*t**6 - 4327680*t**5 -
1108416*t**4 - 251136*t**3 - 48576*t**2 - 7632*t - 288 + (294912*t**16 -
98304*t**15 - 294912*t**14 - 13824*t**13 - 169984*t**12 - 38784*t**11 +
19072*t**10 + 2584*t**9 + 3608*t**8 + 822*t**7 - 226*t**6 - 37*t**5
- 16*t**4 - 3*t**3)*Dt**5 + (176947200*t**13 - 122683392*t**12 -
92676096*t**11 - 80220672*t**10 - 125647872*t**9 - 48731904*t**8 -
8485248*t**7 - 2009184*t**6 + 310944*t**5 + 171456*t**4 + 52128*t**3 +
11352*t**2 + 720*t - 144)*Dt**2 + (176947200*t**12 - 143917056*t**11
- 68567040*t**10 - 100905984*t**9 - 131112960*t**8 - 56203776*t**7 -
15108864*t**6 - 3825024*t**5 - 464256*t**4 - 41136*t**3 + 19440*t**2 +
6984*t + 864)*Dt + (58982400*t**14 - 33816576*t**13 - 39591936*t**12
- 19198464*t**11 - 39716864*t**10 - 13391616*t**9 - 606080*t**8 -
114208*t**7 + 344160*t**6 + 118928*t**5 + 15424*t**4 + 3144*t**3 -
272*t**2 - 156*t)*Dt**3 + (7372800*t**15 - 3342336*t**14 - 6119424*t**13 -
1394688*t**12 - 4639744*t**11 - 1308288*t**10 + 201728*t**9 + 38608*t**8 +
69288*t**7 + 19864*t**6 - 490*t**5 - 42*t**4 - 172*t**3 - 42*t**2)*Dt**4 )

dop[16, 1, 1] = ( 680140800*t**14 + 1617076224*t**13 +
1810280448*t**12 + 607414272*t**11 - 1026155520*t**10 - 1794541056*t**9
- 1579041792*t**8 - 920561280*t**7 - 381792480*t**6 - 110251968*t**5
- 21776496*t**4 - 3222144*t**3 - 336960*t**2 - 23616*t - 432 +
(5667840*t**19 + 20109312*t**18 + 28356864*t**17 + 17725952*t**16 -
2727808*t**15 - 14903680*t**14 - 13628128*t**13 - 7143840*t**12 -
2200052*t**11 - 271904*t**10 + 83757*t**9 + 43688*t**8 + 7231*t**7 -
16*t**6 - 213*t**5 - 24*t**4 - 3*t**3)*Dt**5 + (3400704000*t**16 +
9677463552*t**15 + 12015332352*t**14 + 6056352768*t**13 - 3745480704*t**12
- 9066036480*t**11 - 8261678208*t**10 - 4745464128*t**9 - 1840954848*t**8
- 470314704*t**7 - 65540964*t**6 - 2041416*t**5 + 1229628*t**4 +
258336*t**3 + 28692*t**2 + 792*t - 108)*Dt**2 + (3400704000*t**15 +
8881422336*t**14 + 10496507904*t**13 + 4537731072*t**12 - 4456233984*t**11
- 9049603584*t**10 - 8131117056*t**9 - 4725975936*t**8 - 1908983520*t**7
- 527103072*t**6 - 92579328*t**5 - 10334544*t**4 - 392112*t**3 +
68976*t**2 + 17424*t + 864)*Dt + (1133568000*t**17 + 3491168256*t**16 +
4535958528*t**15 + 2528164864*t**14 - 1007529984*t**13 - 3013656832*t**12
- 2767484032*t**11 - 1557239104*t**10 - 571340832*t**9 - 128930992*t**8 -
10246836*t**7 + 2531992*t**6 + 935788*t**5 + 121672*t**4 + 5868*t**3
- 384*t**2 - 132*t)*Dt**3 + (141696000*t**18 + 469564416*t**17 +
636422400*t**16 + 379738112*t**15 - 96318080*t**14 - 374723840*t**13 -
344541600*t**12 - 188132576*t**11 - 64055524*t**10 - 11768172*t**9 +
368705*t**8 + 725206*t**7 + 165335*t**6 + 12668*t**5 - 1245*t**4 -
246*t**3 - 39*t**2)*Dt**4 )

dop[17, 1, 1] = ( 18*t + 6 + (27*t**3 + 15*t**2 - 6*t)*Dt**2 +
(54*t**2 + 24*t - 6)*Dt + (3*t**4 + 2*t**3 - t**2)*Dt**3 )

dop[18, 1, 1] = ( 72*t + 12 + (108*t**3 + 30*t**2 - 6*t)*Dt**2 +
(12*t**4 + 4*t**3 - t**2)*Dt**3 + (216*t**2 + 48*t - 6)*Dt )

dop[19, 1, 1] = ( 768*t**2 + 864*t + 72 + (64*t**6 + 48*t**5 -
4*t**4 - 3*t**3)*Dt**4 + (3456*t**4 + 3120*t**3 - 12*t**2 - 108*t)*Dt**2 +
(3840*t**3 + 3840*t**2 + 144*t - 72)*Dt + (896*t**5 + 736*t**4 - 32*t**3 -
36*t**2)*Dt**3 )

aux_dop = (
    ( 171286399549440000*t**20
    + 2638485953445888000*t**19
    + 19331169764297932800*t**18
    + 95547038245909954560*t**17
    + 305922196246028156928*t**16
    + 250620273081532809216*t**15
    - 2109325213532559730176*t**14
    - 7145884271499315095808*t**13
    - 3430293600467311627698*t**12
    + 20682911567733215854068*t**11
    + 47990562381910083377106*t**10
    + 78674260785883006097916*t**9
    + 156502192094889767842500*t**8
    + 231740151845288523481116*t**7
    + 201550717065636967782456*t**6
    + 109094259844540767186468*t**5
    + 46937243865521456982810*t**4
    + 11205839292077098881648*t**3
    - 601419249399863477178*t**2
    + 361456534097310660768*t
    - 11410096578718039740)
+   ( 2226723194142720000*t**21
    + 35748717973733376000*t**20
    + 272004146532974592000*t**19
    + 1385814067053110231040*t**18
    + 4750869512437585477632*t**17
    + 6491619741541967069184*t**16
    - 20930172847184317008384*t**15
    - 99865645807353716063232* t**14
    - 94203810665451596781882*t**13
    + 250496473567072068898632*t**12
    + 819756706073592525484554*t**11
    + 1244053067847876051182784*t**10
    + 1605471860922878957020740*t**9
    + 1858802515746125008520124*t**8
    + 1555098899837038744278384*t**7
    + 870965837475739000453332*t**6
    + 371928114059772941512650*t **5
    + 111125056454683385893452*t**4
    + 12722529787944270840918*t**3
    + 5066358677363468512332*t**2
    + 454433524450107373980*t
    + 73842306926255686800)*Dt
+   ( 4881662387159040000*t**22
    + 80753049735266304000*t**21
    + 629405519148063129600*t**20
    + 3255761659317458042880*t**19
    + 11552806180152728469504*t** 18
    + 19283253514830100021248*t**17
    - 33248338221487272438528*t**16
    - 217892506229074630571904*t**15
    - 271811139427074116734479*t**14
    + 450690310348484671418814*t**13
    + 1849575994108796733911568*t**12
    + 2851291232570334753432288*t**11
    + 3264271481752484577090075*t**10
    + 3417228683787005624529288* t**9
    + 2815825484330035867357848*t**8
    + 1620282647986272503444484*t**7
    + 713280447784092767136615*t**6
    + 231725308173375223947474*t**5
    + 34052911521663351312936*t**4
    + 9507737103734602795644*t**3
    - 2476374668076035988675*t**2
    - 849186529651940398200*t)*Dt**2
+   ( 3568466657280000000*t**23
    + 60607789352878080000*t**22
    + 481880675918807040000*t**21
    + 2518387330058645667840*t**20
    + 9135487122412799803392*t**19
    + 17195487476874947149824*t**18
    - 15742726783931920414464*t**17
    - 154414229634782039308032*t**16
    - 232860763488679684519257*t**15
    + 253989014873059993363452*t**14
    + 1320153225125680773206514*t**13
    + 2090065454667377615740554*t**12
    + 2261726431419225885336495*t**11
    + 2228988872540629548125064*t**10
    + 1801556701939225599554484*t**9
    + 1029322603680496535767752*t**8
    + 419184263491501550940405*t**7
    + 93410473794219576003672*t**6
    - 30381530117047380669222*t**5
    - 25721411491709226834498*t**4
    - 15248703228467129341515*t**3
    - 4072403226983001127020*t**2)*Dt**3
+   ( 1106224663756800000*t**24
    + 19256387536158720000*t**23
    + 155854909499965440000*t**22
    + 820768926988031754240*t**21
    + 3020814626208274778112*t**20
    + 6164339216610525179904*t**19
    - 2449275739278857645184*t**18
    - 45832064756814927087552*t**17
    - 80318434574942982693432*t**16
    + 56165563325296843771032*t**15
    + 391193059247216199680904*t**14
    + 640022510382155408250144*t**13
    + 670214676801655249559280*t**12
    + 619661933236562324459904*t**11
    + 459901226012972655621744*t**10
    + 222539574832726591922832*t**9
    + 48716923385181953894760*t**8
    - 36328880339408087086968*t**7
    - 56461452018421838909112*t**6
    - 32807014115021290700208*t**5
    - 13424480013754209443040*t**4
    - 3475511245995767658720*t**3)*Dt**4
+   ( 158439919583232000*t**25
    + 2823482633512550400*t**24
    + 23232099174477987840*t**23
    + 123066073706197155840*t**22
    + 456994176870853896192*t**21
    + 986294464354070581248*t**20
    - 40851351958749464448*t**19
    - 6257517734649635487360*t**18
    - 12453050537819654635080*t**17
    + 4973247815417675742096*t**16
    + 52940756439371347553808*t**15
    + 90134498420892294542640*t**14
    + 92840779759664202045048*t**13
    + 78982085750700931593984*t**12
    + 46048936082557508210016*t**11
    + 7342300324280009410272*t**10
    - 14197463182376967351960*t**9
    - 21899606469934233857328*t**8
    - 19992291410589627258864*t**7
    - 10726942654794782574864*t**6
    - 3902885744775768223128*t**5
    - 1002286246012377188832*t**4)*Dt**5
+   ( 10229604417536000*t**26
    + 186465670450380800*t**25
    + 1558247276799590400*t**24
    + 8291985290526720000*t**23
    + 30926861387906079744*t**22
    + 69352122316429357056*t**21
    + 14433073350071466240*t**20
    - 386261218806867357696*t**19
    - 858449497858188502320*t**18
    + 130797285809167964736*t**17
    + 3204661566080663640288*t**16
    + 5731598104156998387072*t**15
    + 5913428713345357471440*t**14
    + 4528670768491283936640*t**13
    + 1299386979692729270208*t**12
    - 1886679250327506243456*t**11
    - 2957766894527887422864*t**10
    - 3000496027940943666624*t**9
    - 2420888498530584285600*t**8
    - 1253326746737469420288*t**7
    - 430086384539334981648*t**6
    - 110271178343208492288*t**5)*Dt**6
+   ( 237897777152000*t**27
    + 4432499782451200*t**26
    + 37590867534151680*t**25
    + 200731571750830080*t**24
    + 749073871711429632*t**23
    + 1721213540464951296*t**22
    + 653298204354250752 *t**21
    - 8678888448500772864*t**20
    - 21218173039262960784*t**19
    - 1318058326269457920*t**18
    + 69416022010979773728*t**17
    + 132117007259649386016*t**16
    + 139651813549919190384*t**15
    + 95121191333814111360*t**14
    - 16866880701113776320*t**13
    - 117120864788249018112*t**12
    - 132153632111141165232*t**11
    - 118665540330140802240*t**10
    - 91264771850857236576*t**9
    - 46315245385604508192*t**8
    - 15345679965249398832*t**7
    - 3938256369400303296*t**6)*Dt**7
)

def test_ct(i, xx, yy, infolevel=0):
    A, (Du,Dv,Dt) = OreAlgebra(Frac(ZZ['t,u,v']), 'Du', 'Dv', 'Dt').objgens()
    q = A.base_ring()(rat[i](x=xx, y=yy))
    ct1, _ = A.ideal([q*D - D(q) for D in (Du,Dv,Dt)]).ct(Dv)
    ct2, _ = ct1[0].parent().ideal(ct1).ct(Du, infolevel=infolevel)
    assert len(ct2) == 1
    tel = ct2[0]
    cst = tel//dop[i,xx,yy]
    assert cst == 1 or cst == -1

aux_ini = [ Integer(59)/Integer(36), Integer(547)/Integer(2160),
        Integer(1)/Integer(2)*Integer(229)/Integer(252),
        Integer(1)/Integer(6)*-Integer(20249)/Integer(68040),
        Integer(1)/Integer(24)*Integer(1412134)/Integer(280665),
        Integer(1)/Integer(120)*Integer(1787365)/Integer(2918916),
        Integer(1)/Integer(720)*Integer(194607493)/Integer(2189187) ]
